@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2009  The DOSBox Team
+ *  Copyright (C) 2002-2010  The DOSBox Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-/* $Id: callback.cpp,v 1.40 2009/03/03 18:30:41 c2woody Exp $ */
+/* $Id: callback.cpp,v 1.42 2009-08-23 17:24:54 c2woody Exp $ */
 
 #include <stdlib.h>
 #include <string.h>
@@ -26,7 +26,7 @@
 #include "mem.h"
 #include "cpu.h"
 
-/* CallBack are located at 0xF100:0  (see CB_SEG in callback.h)
+/* CallBack are located at 0xF000:0x1000  (see CB_SEG and CB_SOFFSET in callback.h)
    And they are 16 bytes each and you can define them to behave in certain ways like a
    far return or and IRET
 */
@@ -87,8 +87,8 @@ static Bitu stop_handler(void) {
 
 void CALLBACK_RunRealFar(Bit16u seg,Bit16u off) {
 	reg_sp-=4;
-	mem_writew(SegPhys(ss)+reg_sp,call_stop*CB_SIZE);
-	mem_writew(SegPhys(ss)+reg_sp+2,CB_SEG);
+	mem_writew(SegPhys(ss)+reg_sp,RealOff(CALLBACK_RealPointer(call_stop)));
+	mem_writew(SegPhys(ss)+reg_sp+2,RealSeg(CALLBACK_RealPointer(call_stop)));
 	Bit32u oldeip=reg_eip;
 	Bit16u oldcs=SegValue(cs);
 	reg_eip=off;
@@ -101,7 +101,7 @@ void CALLBACK_RunRealFar(Bit16u seg,Bit16u off) {
 void CALLBACK_RunRealInt(Bit8u intnum) {
 	Bit32u oldeip=reg_eip;
 	Bit16u oldcs=SegValue(cs);
-	reg_eip=(CB_MAX*CB_SIZE)+(intnum*6);
+	reg_eip=CB_SOFFSET+(CB_MAX*CB_SIZE)+(intnum*6);
 	SegSet16(cs,CB_SEG);
 	DOSBOX_RunMachine();
 	reg_eip=oldeip;
@@ -109,15 +109,24 @@ void CALLBACK_RunRealInt(Bit8u intnum) {
 }
 
 void CALLBACK_SZF(bool val) {
-	Bit16u tempf=mem_readw(SegPhys(ss)+reg_sp+4) & 0xFFBF;
-	Bit16u newZF=(val==true) << 6;
-	mem_writew(SegPhys(ss)+reg_sp+4,(tempf | newZF));
+	Bit16u tempf = mem_readw(SegPhys(ss)+reg_sp+4); 
+	if (val) tempf |= FLAG_ZF; 
+	else tempf &= ~FLAG_ZF; 
+	mem_writew(SegPhys(ss)+reg_sp+4,tempf); 
 }
 
 void CALLBACK_SCF(bool val) {
-	Bit16u tempf=mem_readw(SegPhys(ss)+reg_sp+4) & 0xFFFE;
-	Bit16u newCF=(val==true);
-	mem_writew(SegPhys(ss)+reg_sp+4,(tempf | newCF));
+	Bit16u tempf = mem_readw(SegPhys(ss)+reg_sp+4); 
+	if (val) tempf |= FLAG_CF; 
+	else tempf &= ~FLAG_CF; 
+	mem_writew(SegPhys(ss)+reg_sp+4,tempf); 
+}
+
+void CALLBACK_SIF(bool val) {
+	Bit16u tempf = mem_readw(SegPhys(ss)+reg_sp+4); 
+	if (val) tempf |= FLAG_IF; 
+	else tempf &= ~FLAG_IF; 
+	mem_writew(SegPhys(ss)+reg_sp+4,tempf); 
 }
 
 void CALLBACK_SetDescription(Bitu nr, const char* descr) {
@@ -141,7 +150,7 @@ Bitu CALLBACK_SetupExtra(Bitu callback, Bitu type, PhysPt physAddress, bool use_
 		if (use_cb) {
 			phys_writeb(physAddress+0x00,(Bit8u)0xFE);	//GRP 4
 			phys_writeb(physAddress+0x01,(Bit8u)0x38);	//Extra Callback instruction
-			phys_writew(physAddress+0x02, callback);	//The immediate word
+			phys_writew(physAddress+0x02,(Bit16u)callback);	//The immediate word
 			physAddress+=4;
 		}
 		phys_writeb(physAddress+0x00,(Bit8u)0xC3);		//A RETN Instruction
@@ -150,7 +159,7 @@ Bitu CALLBACK_SetupExtra(Bitu callback, Bitu type, PhysPt physAddress, bool use_
 		if (use_cb) {
 			phys_writeb(physAddress+0x00,(Bit8u)0xFE);	//GRP 4
 			phys_writeb(physAddress+0x01,(Bit8u)0x38);	//Extra Callback instruction
-			phys_writew(physAddress+0x02, callback);	//The immediate word
+			phys_writew(physAddress+0x02,(Bit16u)callback);	//The immediate word
 			physAddress+=4;
 		}
 		phys_writeb(physAddress+0x00,(Bit8u)0xCB);		//A RETF Instruction
@@ -159,7 +168,7 @@ Bitu CALLBACK_SetupExtra(Bitu callback, Bitu type, PhysPt physAddress, bool use_
 		if (use_cb) {
 			phys_writeb(physAddress+0x00,(Bit8u)0xFE);	//GRP 4
 			phys_writeb(physAddress+0x01,(Bit8u)0x38);	//Extra Callback instruction
-			phys_writew(physAddress+0x02, callback);	//The immediate word
+			phys_writew(physAddress+0x02,(Bit16u)callback);	//The immediate word
 			physAddress+=4;
 		}
 		phys_writeb(physAddress+0x00,(Bit8u)0xCA);		//A RETF 8 Instruction
@@ -169,7 +178,7 @@ Bitu CALLBACK_SetupExtra(Bitu callback, Bitu type, PhysPt physAddress, bool use_
 		if (use_cb) {
 			phys_writeb(physAddress+0x00,(Bit8u)0xFE);	//GRP 4
 			phys_writeb(physAddress+0x01,(Bit8u)0x38);	//Extra Callback instruction
-			phys_writew(physAddress+0x02,callback);		//The immediate word
+			phys_writew(physAddress+0x02,(Bit16u)callback);		//The immediate word
 			physAddress+=4;
 		}
 		phys_writeb(physAddress+0x00,(Bit8u)0xCF);		//An IRET Instruction
@@ -178,7 +187,7 @@ Bitu CALLBACK_SetupExtra(Bitu callback, Bitu type, PhysPt physAddress, bool use_
 		if (use_cb) {
 			phys_writeb(physAddress+0x00,(Bit8u)0xFE);	//GRP 4
 			phys_writeb(physAddress+0x01,(Bit8u)0x38);	//Extra Callback instruction
-			phys_writew(physAddress+0x02,callback);		//The immediate word
+			phys_writew(physAddress+0x02,(Bit16u)callback);		//The immediate word
 			physAddress+=4;
 		}
 		phys_writeb(physAddress+0x00,(Bit8u)0x66);		//An IRETD Instruction
@@ -189,7 +198,7 @@ Bitu CALLBACK_SetupExtra(Bitu callback, Bitu type, PhysPt physAddress, bool use_
 		if (use_cb) {
 			phys_writeb(physAddress+0x01,(Bit8u)0xFE);	//GRP 4
 			phys_writeb(physAddress+0x02,(Bit8u)0x38);	//Extra Callback instruction
-			phys_writew(physAddress+0x03, callback);	//The immediate word
+			phys_writew(physAddress+0x03,(Bit16u)callback);	//The immediate word
 			physAddress+=4;
 		}
 		phys_writeb(physAddress+0x01,(Bit8u)0xCF);		//An IRET Instruction
@@ -198,7 +207,7 @@ Bitu CALLBACK_SetupExtra(Bitu callback, Bitu type, PhysPt physAddress, bool use_
 		if (use_cb) {
 			phys_writeb(physAddress+0x00,(Bit8u)0xFE);	//GRP 4
 			phys_writeb(physAddress+0x01,(Bit8u)0x38);	//Extra Callback instruction
-			phys_writew(physAddress+0x02,callback);		//The immediate word
+			phys_writew(physAddress+0x02,(Bit16u)callback);		//The immediate word
 			physAddress+=4;
 		}
 		phys_writeb(physAddress+0x00,(Bit8u)0x50);		// push ax
@@ -213,7 +222,7 @@ Bitu CALLBACK_SetupExtra(Bitu callback, Bitu type, PhysPt physAddress, bool use_
 		if (use_cb) {
 			phys_writeb(physAddress+0x00,(Bit8u)0xFE);	//GRP 4
 			phys_writeb(physAddress+0x01,(Bit8u)0x38);	//Extra Callback instruction
-			phys_writew(physAddress+0x02,callback);		//The immediate word
+			phys_writew(physAddress+0x02,(Bit16u)callback);		//The immediate word
 			physAddress+=4;
 		}
 		phys_writeb(physAddress+0x00,(Bit8u)0x50);		// push ax
@@ -238,7 +247,7 @@ Bitu CALLBACK_SetupExtra(Bitu callback, Bitu type, PhysPt physAddress, bool use_
 			phys_writew(physAddress+0x08,(Bit16u)0x0473);	// jc skip
 			phys_writeb(physAddress+0x0a,(Bit8u)0xFE);		//GRP 4
 			phys_writeb(physAddress+0x0b,(Bit8u)0x38);		//Extra Callback instruction
-			phys_writew(physAddress+0x0c,callback);			//The immediate word
+			phys_writew(physAddress+0x0c,(Bit16u)callback);			//The immediate word
 			// jump here to (skip):
 			physAddress+=6;
 		}
@@ -252,7 +261,7 @@ Bitu CALLBACK_SetupExtra(Bitu callback, Bitu type, PhysPt physAddress, bool use_
 		if (use_cb) {
 			phys_writeb(physAddress+0x00,(Bit8u)0xFE);	//GRP 4
 			phys_writeb(physAddress+0x01,(Bit8u)0x38);	//Extra Callback instruction
-			phys_writew(physAddress+0x02,callback);		//The immediate word
+			phys_writew(physAddress+0x02,(Bit16u)callback);		//The immediate word
 			physAddress+=4;
 		}
 		phys_writeb(physAddress+0x00,(Bit8u)0x50);		// push ax
@@ -272,13 +281,13 @@ Bitu CALLBACK_SetupExtra(Bitu callback, Bitu type, PhysPt physAddress, bool use_
 		phys_writeb(physAddress+0x05,(Bit8u)0xfb);		// sti
 		phys_writeb(physAddress+0x06,(Bit8u)0xFE);		//GRP 4
 		phys_writeb(physAddress+0x07,(Bit8u)0x38);		//Extra Callback instruction
-		phys_writew(physAddress+0x08,callback);			//The immediate word
+		phys_writew(physAddress+0x08,(Bit16u)callback);			//The immediate word
 		return 0x0a;
 	case CB_IRQ12_RET:	// ps2 mouse int74 return
 		if (use_cb) {
 			phys_writeb(physAddress+0x00,(Bit8u)0xFE);	//GRP 4
 			phys_writeb(physAddress+0x01,(Bit8u)0x38);	//Extra Callback instruction
-			phys_writew(physAddress+0x02,callback);		//The immediate word
+			phys_writew(physAddress+0x02,(Bit16u)callback);		//The immediate word
 			physAddress+=4;
 		}
 		phys_writeb(physAddress+0x00,(Bit8u)0xfa);		// cli
@@ -298,7 +307,7 @@ Bitu CALLBACK_SetupExtra(Bitu callback, Bitu type, PhysPt physAddress, bool use_
 			phys_writew(physAddress+0x05,(Bit16u)0x0674);	// je skip
 			phys_writeb(physAddress+0x07,(Bit8u)0xFE);		//GRP 4
 			phys_writeb(physAddress+0x08,(Bit8u)0x38);		//Extra Callback instruction
-			phys_writew(physAddress+0x09,callback);			//The immediate word
+			phys_writew(physAddress+0x09,(Bit16u)callback);			//The immediate word
 			physAddress+=4;
 		} else {
 			phys_writew(physAddress+0x05,(Bit16u)0x0274);	// je skip
@@ -318,7 +327,7 @@ Bitu CALLBACK_SetupExtra(Bitu callback, Bitu type, PhysPt physAddress, bool use_
 		if (use_cb) {
 			phys_writeb(physAddress+0x00,(Bit8u)0xFE);	//GRP 4
 			phys_writeb(physAddress+0x01,(Bit8u)0x38);	//Extra Callback instruction
-			phys_writew(physAddress+0x02,callback);		//The immediate word
+			phys_writew(physAddress+0x02,(Bit16u)callback);		//The immediate word
 			physAddress+=4;
 		}
 		phys_writeb(physAddress+0x00,(Bit8u)0xCF);		//An IRET Instruction
@@ -328,7 +337,7 @@ Bitu CALLBACK_SetupExtra(Bitu callback, Bitu type, PhysPt physAddress, bool use_
 		if (use_cb) {
 			phys_writeb(physAddress+0x01,(Bit8u)0xFE);	//GRP 4
 			phys_writeb(physAddress+0x02,(Bit8u)0x38);	//Extra Callback instruction
-			phys_writew(physAddress+0x03, callback);	//The immediate word
+			phys_writew(physAddress+0x03,(Bit16u)callback);	//The immediate word
 			physAddress+=4;
 		}
 		phys_writeb(physAddress+0x01,(Bit8u)0xCF);		//An IRET Instruction
@@ -339,7 +348,7 @@ Bitu CALLBACK_SetupExtra(Bitu callback, Bitu type, PhysPt physAddress, bool use_
 		if (use_cb) {
 			phys_writeb(physAddress+0x00,(Bit8u)0xFE);	//GRP 4
 			phys_writeb(physAddress+0x01,(Bit8u)0x38);	//Extra Callback instruction
-			phys_writew(physAddress+0x02,callback);		//The immediate word
+			phys_writew(physAddress+0x02,(Bit16u)callback);		//The immediate word
 			physAddress+=4;
 		}
 		phys_writeb(physAddress+0x00,(Bit8u)0x50);		// push ax
@@ -357,7 +366,7 @@ Bitu CALLBACK_SetupExtra(Bitu callback, Bitu type, PhysPt physAddress, bool use_
 		if (use_cb) {
 			phys_writeb(physAddress+0x05,(Bit8u)0xFE);	//GRP 4
 			phys_writeb(physAddress+0x06,(Bit8u)0x38);	//Extra Callback instruction
-			phys_writew(physAddress+0x07,callback);		//The immediate word
+			phys_writew(physAddress+0x07,(Bit16u)callback);		//The immediate word
 			physAddress+=4;
 		}
 		phys_writeb(physAddress+0x05,(Bit8u)0xCB);		//A RETF Instruction
@@ -366,7 +375,7 @@ Bitu CALLBACK_SetupExtra(Bitu callback, Bitu type, PhysPt physAddress, bool use_
 		if (use_cb) {
 			phys_writeb(physAddress+0x00,(Bit8u)0xFE);	//GRP 4
 			phys_writeb(physAddress+0x01,(Bit8u)0x38);	//Extra Callback instruction
-			phys_writew(physAddress+0x02,callback);		//The immediate word
+			phys_writew(physAddress+0x02,(Bit16u)callback);		//The immediate word
 			physAddress+=4;
 		}
 		phys_writeb(physAddress+0x00,(Bit8u)0x50);		// push ax
@@ -388,7 +397,7 @@ Bitu CALLBACK_SetupExtra(Bitu callback, Bitu type, PhysPt physAddress, bool use_
 		phys_writeb(physAddress+0x06,(Bit8u)0x60);		// pusha
 		phys_writeb(physAddress+0x07,(Bit8u)0xFE);		//GRP 4
 		phys_writeb(physAddress+0x08,(Bit8u)0x38);		//Extra Callback instruction
-		phys_writew(physAddress+0x09,callback);			//The immediate word
+		phys_writew(physAddress+0x09,(Bit16u)callback);	//The immediate word
 		phys_writeb(physAddress+0x0b,(Bit8u)0xCB);		//A RETF Instruction
 		return 0x0c;
 	case CB_IPXESR_RET:		// IPX ESR return
@@ -409,12 +418,18 @@ Bitu CALLBACK_SetupExtra(Bitu callback, Bitu type, PhysPt physAddress, bool use_
 		if (use_cb) {
 			phys_writeb(physAddress+0x01,(Bit8u)0xFE);	//GRP 4
 			phys_writeb(physAddress+0x02,(Bit8u)0x38);	//Extra Callback instruction
-			phys_writew(physAddress+0x03, callback);	//The immediate word
+			phys_writew(physAddress+0x03,(Bit16u)callback);	//The immediate word
 			physAddress+=4;
 		}
 		phys_writeb(physAddress+0x01,(Bit8u)0xCF);		//An IRET Instruction
 		phys_writeb(physAddress+0x02,(Bit8u)0xCB);		//A RETF Instruction
-		return (use_cb?7:3);
+		phys_writeb(physAddress+0x03,(Bit8u)0x51);		// push cx
+		phys_writeb(physAddress+0x04,(Bit8u)0xB9);		// mov cx,
+		phys_writew(physAddress+0x05,(Bit16u)0x0140);		// 0x140
+		phys_writew(physAddress+0x07,(Bit16u)0xFEE2);		// loop $-2
+		phys_writeb(physAddress+0x09,(Bit8u)0x59);		// pop cx
+		phys_writeb(physAddress+0x0A,(Bit8u)0xCF);		//An IRET Instruction
+		return (use_cb?15:11);
 
 	default:
 		E_Exit("CALLBACK:Setup:Illegal type %d",type);
@@ -502,7 +517,7 @@ void CALLBACK_HandlerObject::Set_RealVec(Bit8u vec){
 	} else E_Exit ("double usage of vector handler");
 }
 
-void CALLBACK_Init(Section* sec) {
+void CALLBACK_Init(Section* /*sec*/) {
 	Bitu i;
 	for (i=0;i<CB_MAX;i++) {
 		CallBack_Handlers[i]=&illegal_handler;
@@ -514,7 +529,7 @@ void CALLBACK_Init(Section* sec) {
 	CALLBACK_SetDescription(call_stop,"stop");
 	phys_writeb(CALLBACK_PhysPointer(call_stop)+0,0xFE);
 	phys_writeb(CALLBACK_PhysPointer(call_stop)+1,0x38);
-	phys_writew(CALLBACK_PhysPointer(call_stop)+2,call_stop);
+	phys_writew(CALLBACK_PhysPointer(call_stop)+2,(Bit16u)call_stop);
 
 	/* Setup the idle handler */
 	call_idle=CALLBACK_Allocate();
@@ -523,7 +538,7 @@ void CALLBACK_Init(Section* sec) {
 	for (i=0;i<=11;i++) phys_writeb(CALLBACK_PhysPointer(call_idle)+i,0x90);
 	phys_writeb(CALLBACK_PhysPointer(call_idle)+12,0xFE);
 	phys_writeb(CALLBACK_PhysPointer(call_idle)+13,0x38);
-	phys_writew(CALLBACK_PhysPointer(call_idle)+14,call_idle);
+	phys_writew(CALLBACK_PhysPointer(call_idle)+14,(Bit16u)call_idle);
 
 	/* Default handlers for unhandled interrupts that have to be non-null */
 	call_default=CALLBACK_Allocate();
@@ -531,18 +546,21 @@ void CALLBACK_Init(Section* sec) {
 	call_default2=CALLBACK_Allocate();
 	CALLBACK_Setup(call_default2,&default_handler,CB_IRET,"default");
    
-	/* Only setup default handler for first half of interrupt table */
-	for (i=0;i<0x40;i++) {
-		real_writed(0,i*4,CALLBACK_RealPointer(call_default));
+	/* Only setup default handler for first part of interrupt table */
+	for (Bit16u ct=0;ct<0x60;ct++) {
+		real_writed(0,ct*4,CALLBACK_RealPointer(call_default));
+	}
+	for (Bit16u ct=0x68;ct<0x70;ct++) {
+		real_writed(0,ct*4,CALLBACK_RealPointer(call_default));
 	}
 	/* Setup block of 0xCD 0xxx instructions */
-	PhysPt rint_base=(CB_SEG << 4)+CB_MAX*CB_SIZE;
+	PhysPt rint_base=CALLBACK_GetBase()+CB_MAX*CB_SIZE;
 	for (i=0;i<=0xff;i++) {
 		phys_writeb(rint_base,0xCD);
-		phys_writeb(rint_base+1,i);
+		phys_writeb(rint_base+1,(Bit8u)i);
 		phys_writeb(rint_base+2,0xFE);
 		phys_writeb(rint_base+3,0x38);
-		phys_writew(rint_base+4,call_stop);
+		phys_writew(rint_base+4,(Bit16u)call_stop);
 		rint_base+=6;
 
 	}
