@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2009  The DOSBox Team
+ *  Copyright (C) 2002-2010  The DOSBox Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-/* $Id: core_dynrec.cpp,v 1.12 2009/05/27 09:15:41 qbix79 Exp $ */
+/* $Id: core_dynrec.cpp,v 1.15 2009-08-02 16:52:33 c2woody Exp $ */
 
 #include "dosbox.h"
 
@@ -51,6 +51,7 @@
 #include "paging.h"
 #include "inout.h"
 #include "lazyflags.h"
+#include "pic.h"
 
 #define CACHE_MAXSIZE	(4096*2)
 #define CACHE_TOTAL		(1024*1024*8)
@@ -99,9 +100,9 @@
 #define DRCD_SEG_PHYS(seg) (&Segs.phys[seg])
 
 // access to an 8bit general register
-#define DRCD_REG_BYTE(reg,idx) (&cpu_regs.regs[reg].byte[idx])
+#define DRCD_REG_BYTE(reg,idx) (&cpu_regs.regs[reg].byte[idx?BH_INDEX:BL_INDEX])
 // access to  16/32bit general registers
-#define DRCD_REG_WORD(reg,dwrd) ((dwrd)?((void*)(&cpu_regs.regs[reg].dword)):((void*)(&cpu_regs.regs[reg].word)))
+#define DRCD_REG_WORD(reg,dwrd) ((dwrd)?((void*)(&cpu_regs.regs[reg].dword[DW_INDEX])):((void*)(&cpu_regs.regs[reg].word[W_INDEX])))
 
 
 enum BlockReturn {
@@ -139,6 +140,7 @@ static struct {
 #define X86_64		0x02
 #define MIPSEL		0x03
 #define ARMV4LE		0x04
+#define POWERPC		0x04
 
 #if C_TARGETCPU == X86_64
 #include "core_dynrec/risc_x64.h"
@@ -148,6 +150,8 @@ static struct {
 #include "core_dynrec/risc_mipsel32.h"
 #elif C_TARGETCPU == ARMV4LE
 #include "core_dynrec/risc_armv4le.h"
+#elif C_TARGETCPU == POWERPC
+#include "core_dynrec/risc_ppc.h"
 #endif
 
 #include "core_dynrec/decoder.h"
@@ -233,7 +237,10 @@ run_block:
 #if C_HEAVY_DEBUG
 			if (DEBUG_HeavyIsBreakpoint()) return debugCallback;
 #endif
-			if (!GETFLAG(TF)) break;
+			if (!GETFLAG(TF)) {
+				if (GETFLAG(IF) && PIC_IRQCheck) return CBRET_NONE;
+				break;
+			}
 			// trapflag is set, switch to the trap-aware decoder
 			cpudecoder=CPU_Core_Dynrec_Trap_Run;
 			return CBRET_NONE;
