@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2009  The DOSBox Team
+ *  Copyright (C) 2002-2010  The DOSBox Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-/* $Id: decoder.h,v 1.6 2009/05/27 09:15:41 qbix79 Exp $ */
+/* $Id: decoder.h,v 1.8 2009-10-18 17:52:10 c2woody Exp $ */
 
 
 #include "decoder_basic.h"
@@ -566,7 +566,16 @@ restart_prefix:
 			if (dyn_grp4_eb()) goto finish_block;
 			break;
 		case 0xff:
-			if (dyn_grp4_ev()) goto core_close_block;
+			switch (dyn_grp4_ev()) {
+			case 0:
+				break;
+			case 1:
+				goto core_close_block;
+			case 2:
+				goto illegalopcode;
+			default:
+				break;
+			}
 			break;
 
 		default:
@@ -574,8 +583,12 @@ restart_prefix:
 			goto illegalopcode;
 		}
 	}
-	// normal exit because the maximal number of opcodes has been reached
+	// link to next block because the maximal number of opcodes has been reached
 	dyn_set_eip_end();
+	dyn_reduce_cycles();
+	gen_jmp_ptr(&decode.block->link[0].to,offsetof(CacheBlockDynRec,cache.start));
+	dyn_closeblock();
+    goto finish_block;
 core_close_block:
 	dyn_reduce_cycles();
 	dyn_return(BR_Normal);
