@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2009  The DOSBox Team
+ *  Copyright (C) 2002-2010  The DOSBox Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-/* $Id: serialport.cpp,v 1.12 2009/05/27 09:15:41 qbix79 Exp $ */
+/* $Id: serialport.cpp,v 1.14 2009-10-01 17:25:28 h-a-l-9000 Exp $ */
 
 #include <string.h>
 #include <ctype.h>
@@ -30,15 +30,14 @@
 #include "callback.h"				// CALLBACK_Idle
 
 #include "serialport.h"
-#include "directserial_win32.h"
-#include "directserial_posix.h"
-#include "directserial_os2.h"
+#include "directserial.h"
 #include "serialdummy.h"
 #include "softmodem.h"
 #include "nullmodem.h"
 
 #include "cpu.h"
 
+#define LOG_SER(x) log_ser 
 
 bool device_COM::Read(Bit8u * data,Bit16u * size) {
 	// DTR + RTS on
@@ -98,97 +97,140 @@ device_COM::~device_COM() {
 CSerial* serialports[4] ={0,0,0,0};
 
 static Bitu SERIAL_Read (Bitu port, Bitu iolen) {
-	for(Bitu i = 0; i < 4; i++) {
-		if(serial_baseaddr[i]==(port&0xfff8) && (serialports[i]!=0)) {
-			Bitu retval=0xff;
-			switch (port & 0x7) {
-				case RHR_OFFSET:
-					retval = serialports[i]->Read_RHR();
-					break;
-				case IER_OFFSET:
-					retval = serialports[i]->Read_IER();
-					break;
-				case ISR_OFFSET:
-					retval = serialports[i]->Read_ISR();
-					break;
-				case LCR_OFFSET:
-					retval = serialports[i]->Read_LCR();
-					break;
-				case MCR_OFFSET:
-					retval = serialports[i]->Read_MCR();
-					break;
-				case LSR_OFFSET:
-					retval = serialports[i]->Read_LSR();
-					break;
-				case MSR_OFFSET:
-					retval = serialports[i]->Read_MSR();
-					break;
-				case SPR_OFFSET:
-					retval = serialports[i]->Read_SPR();
-					break;
-			}
+	Bitu i;
+	Bitu retval;
+	Bitu index = port & 0x7;
+	switch(port&0xff8) {
+		case 0x3f8: i=0; break;
+		case 0x2f8: i=1; break;
+		case 0x3e8: i=2; break;
+		case 0x2e8: i=3; break;
+		default: return 0xff;
+	}
+	if(serialports[i]==0) return 0xff;
+
+	switch (index) {
+		case RHR_OFFSET:
+			retval = serialports[i]->Read_RHR();
+			break;
+		case IER_OFFSET:
+			retval = serialports[i]->Read_IER();
+			break;
+		case ISR_OFFSET:
+			retval = serialports[i]->Read_ISR();
+			break;
+		case LCR_OFFSET:
+			retval = serialports[i]->Read_LCR();
+			break;
+		case MCR_OFFSET:
+			retval = serialports[i]->Read_MCR();
+			break;
+		case LSR_OFFSET:
+			retval = serialports[i]->Read_LSR();
+			break;
+		case MSR_OFFSET:
+			retval = serialports[i]->Read_MSR();
+			break;
+		case SPR_OFFSET:
+			retval = serialports[i]->Read_SPR();
+			break;
+	}
 
 #if SERIAL_DEBUG
-			const char* const dbgtext[]=
-				{"RHR","IER","ISR","LCR","MCR","LSR","MSR","SPR"};
-			if(serialports[i]->dbg_register)
-				fprintf(serialports[i]->debugfp,"%12.3f read 0x%x from %s.\r\n",
-					PIC_FullIndex(),retval,dbgtext[port&0x7]);
-#endif
-
-			return retval;	
-		}
+	const char* const dbgtext[]=
+		{"RHR","IER","ISR","LCR","MCR","LSR","MSR","SPR","DLL","DLM"};
+	if(serialports[i]->dbg_register) {
+		if((index<2) && ((serialports[i]->LCR)&LCR_DIVISOR_Enable_MASK))
+			index += 8;
+		serialports[i]->log_ser(serialports[i]->dbg_register,
+			"read  0x%2x from %s.",retval,dbgtext[index]);
 	}
-	return 0xff;
+#endif
+	return retval;	
 }
 static void SERIAL_Write (Bitu port, Bitu val, Bitu) {
+	Bitu i;
+	Bitu index = port & 0x7;
+	switch(port&0xff8) {
+		case 0x3f8: i=0; break;
+		case 0x2f8: i=1; break;
+		case 0x3e8: i=2; break;
+		case 0x2e8: i=3; break;
+		default: return;
+	}
+	if(serialports[i]==0) return;
 	
-	for(Bitu i = 0; i < 4; i++) {
-		if(serial_baseaddr[i]==(port&0xfff8) && serialports[i]) {
-
 #if SERIAL_DEBUG
-		const char* const dbgtext[]={"THR","IER","FCR","LCR","MCR","!LSR","MSR","SPR"};
-		if(serialports[i]->dbg_register)
-			fprintf(serialports[i]->debugfp,"%12.3f write 0x%x to %s.\r\n",
-				PIC_FullIndex(),val,dbgtext[port&0x7]);
-#endif
-
-			switch (port & 0x7) {
-				case THR_OFFSET:
-					serialports[i]->Write_THR (val);
-					return;
-				case IER_OFFSET:
-					serialports[i]->Write_IER (val);
-					return;
-				case FCR_OFFSET:
-					serialports[i]->Write_FCR (val);
-					return;
-				case LCR_OFFSET:
-					serialports[i]->Write_LCR (val);
-					return;
-				case MCR_OFFSET:
-					serialports[i]->Write_MCR (val);
-					return;
-				case MSR_OFFSET:
-					serialports[i]->Write_MSR (val);
-					return;
-				case SPR_OFFSET:
-					serialports[i]->Write_SPR (val);
-					return;
-				default:
-					serialports[i]->Write_reserved (val, port & 0x7);
-			}
+		const char* const dbgtext[]={"THR","IER","FCR",
+			"LCR","MCR","!LSR","MSR","SPR","DLL","DLM"};
+		if(serialports[i]->dbg_register) {
+			Bitu debugindex=index;
+			if((index<2) && ((serialports[i]->LCR)&LCR_DIVISOR_Enable_MASK))
+				debugindex += 8;
+			serialports[i]->log_ser(serialports[i]->dbg_register,
+				"write 0x%2x to %s.",val,dbgtext[debugindex]);
 		}
+#endif
+	switch (index) {
+		case THR_OFFSET:
+			serialports[i]->Write_THR (val);
+			return;
+		case IER_OFFSET:
+			serialports[i]->Write_IER (val);
+			return;
+		case FCR_OFFSET:
+			serialports[i]->Write_FCR (val);
+			return;
+		case LCR_OFFSET:
+			serialports[i]->Write_LCR (val);
+			return;
+		case MCR_OFFSET:
+			serialports[i]->Write_MCR (val);
+			return;
+		case MSR_OFFSET:
+			serialports[i]->Write_MSR (val);
+			return;
+		case SPR_OFFSET:
+			serialports[i]->Write_SPR (val);
+			return;
+		default:
+			serialports[i]->Write_reserved (val, port & 0x7);
 	}
 }
+#if SERIAL_DEBUG
+void CSerial::log_ser(bool active, char const* format,...) {
+	if(active) {
+		// copied from DEBUG_SHOWMSG
+		char buf[512];
+		buf[0]=0;
+		sprintf(buf,"%12.3f [% 7u] ",PIC_FullIndex(), SDL_GetTicks());
+		va_list msg;
+		va_start(msg,format);
+		vsprintf(buf+strlen(buf),format,msg);
+		va_end(msg);
+		// Add newline if not present
+		Bitu len=strlen(buf);
+		if(buf[len-1]!='\n') strcat(buf,"\r\n");
+		fputs(buf,debugfp);
+	}
+}
+#endif
 
 void CSerial::changeLineProperties() {
 	// update the event wait time
+	float bitlen;
 
-	float bitlen = (1000.0f/115200.0f)*(float)baud_divider;
+	if(baud_divider==0) bitlen=(1000.0f/115200.0f);
+	else bitlen = (1000.0f/115200.0f)*(float)baud_divider;
 	bytetime=bitlen*(float)(1+5+1);		// startbit + minimum length + stopbit
 	bytetime+= bitlen*(float)(LCR&0x3); // databits
 	if(LCR&0x4) bytetime+=bitlen;		// stopbit
+
+#if SERIAL_DEBUG
+	const char* const dbgtext[]={"none","odd","none","even","none","mark","none","space"};
+	log_ser(dbg_serialtraffic,"New COM parameters: baudrate %5.0f, parity %s, wordlen %d, stopbits %d",
+		1.0/bitlen*1000.0f,dbgtext[(LCR&0x38)>>3],(LCR&0x3)+5,((LCR&0x4)>>2)+1);
+#endif	
 	updatePortConfig (baud_divider, LCR);
 }
 
@@ -212,35 +254,38 @@ void CSerial::handleEvent(Bit16u type) {
 		case SERIAL_TX_LOOPBACK_EVENT: {
 
 #if SERIAL_DEBUG
-			if(dbg_serialtraffic)
-				fprintf(debugfp,loopback_data<0x10? "%12.3f tx 0x%02x (%u) (loopback)\r\n":
-												    "%12.3f tx 0x%02x (%c) (loopback)\r\n",
-					PIC_FullIndex(),loopback_data,
-					loopback_data);
+			log_ser(dbg_serialtraffic,loopback_data<0x10?
+				"tx 0x%02x (%u) (loopback)":"tx 0x%02x (%c) (loopback)",
+				loopback_data, loopback_data);
 #endif
-
 			receiveByte (loopback_data);
 			ByteTransmitted ();
 			break;
 		}
 		case SERIAL_THR_LOOPBACK_EVENT: {
+			loopback_data=txfifo->probeByte();
 			ByteTransmitting();
-			loopback_data=THR;
 			setEvent(SERIAL_TX_LOOPBACK_EVENT,bytetime);	
 			break;
 		}
 		case SERIAL_ERRMSG_EVENT: {
-			LOG_MSG("Serial%d: Errors occured: "\
-				"Framing %d, Parity %d, Overrun %d (IF0:%d), Break %d", COMNUMBER,
-				framingErrors, parityErrors, overrunErrors, overrunIF0, breakErrors);
+			LOG_MSG("Serial%d: Errors: "\
+				"Framing %d, Parity %d, Overrun RX:%d (IF0:%d), TX:%d, Break %d",
+				COMNUMBER, framingErrors, parityErrors, overrunErrors,
+				overrunIF0,txOverrunErrors, breakErrors);
 			errormsg_pending=false;
 			framingErrors=0;
 			parityErrors=0;
 			overrunErrors=0;
+			txOverrunErrors=0;
 			overrunIF0=0;
 			breakErrors=0;
 			break;					  
 		}
+		case SERIAL_RX_TIMEOUT_EVENT: {
+			rise(TIMEOUT_PRIORITY);
+			break;
+		}
 		default: handleUpperEvent(type);
 	}
 }
@@ -250,20 +295,14 @@ void CSerial::handleEvent(Bit16u type) {
 /*****************************************************************************/
 void CSerial::rise (Bit8u priority) {
 #if SERIAL_DEBUG
-	if(dbg_interrupt)
-	{
-		if(priority&TX_PRIORITY && !(waiting_interrupts&TX_PRIORITY))
-			fprintf(debugfp,"%12.3f tx interrupt on.\r\n",PIC_FullIndex());
-
-		if(priority&RX_PRIORITY && !(waiting_interrupts&RX_PRIORITY))
-			fprintf(debugfp,"%12.3f rx interrupt on.\r\n",PIC_FullIndex());
-
-		if(priority&MSR_PRIORITY && !(waiting_interrupts&MSR_PRIORITY))
-			fprintf(debugfp,"%12.3f msr interrupt on.\r\n",PIC_FullIndex());
-
-		if(priority&ERROR_PRIORITY && !(waiting_interrupts&ERROR_PRIORITY))
-			fprintf(debugfp,"%12.3f error interrupt on.\r\n",PIC_FullIndex());
-	}
+	if(priority&TX_PRIORITY && !(waiting_interrupts&TX_PRIORITY))
+		log_ser(dbg_interrupt,"tx interrupt on.");
+	if(priority&RX_PRIORITY && !(waiting_interrupts&RX_PRIORITY))
+		log_ser(dbg_interrupt,"rx interrupt on.");
+	if(priority&MSR_PRIORITY && !(waiting_interrupts&MSR_PRIORITY))
+		log_ser(dbg_interrupt,"msr interrupt on.");
+	if(priority&TIMEOUT_PRIORITY && !(waiting_interrupts&TIMEOUT_PRIORITY))
+		log_ser(dbg_interrupt,"fifo rx timeout interrupt on.");
 #endif
 	
 	waiting_interrupts |= priority;
@@ -274,23 +313,15 @@ void CSerial::rise (Bit8u priority) {
 void CSerial::clear (Bit8u priority) {
 	
 #if SERIAL_DEBUG
-	if(dbg_interrupt)
-	{
-		if(priority&TX_PRIORITY && (waiting_interrupts&TX_PRIORITY))
-			fprintf(debugfp,"%12.3f tx interrupt off.\r\n",PIC_FullIndex());
-
-		if(priority&RX_PRIORITY && (waiting_interrupts&RX_PRIORITY))
-			fprintf(debugfp,"%12.3f rx interrupt off.\r\n",PIC_FullIndex());
-
-		if(priority&MSR_PRIORITY && (waiting_interrupts&MSR_PRIORITY))
-			fprintf(debugfp,"%12.3f msr interrupt off.\r\n",PIC_FullIndex());
-
-		if(priority&ERROR_PRIORITY && (waiting_interrupts&ERROR_PRIORITY))
-			fprintf(debugfp,"%12.3f error interrupt off.\r\n",PIC_FullIndex());
-	}
+	if(priority&TX_PRIORITY && (waiting_interrupts&TX_PRIORITY))
+		log_ser(dbg_interrupt,"tx interrupt off.");
+	if(priority&RX_PRIORITY && (waiting_interrupts&RX_PRIORITY))
+		log_ser(dbg_interrupt,"rx interrupt off.");
+	if(priority&MSR_PRIORITY && (waiting_interrupts&MSR_PRIORITY))
+		log_ser(dbg_interrupt,"msr interrupt off.");
+	if(priority&ERROR_PRIORITY && (waiting_interrupts&ERROR_PRIORITY))
+		log_ser(dbg_interrupt,"error interrupt off.");
 #endif
-	
-	
 	waiting_interrupts &= (~priority);
 	ComputeInterrupts();
 }
@@ -299,30 +330,30 @@ void CSerial::ComputeInterrupts () {
 
 	Bitu val = IER & waiting_interrupts;
 
-	if (val & ERROR_PRIORITY)	 ISR = ISR_ERROR_VAL;
-	else if (val & RX_PRIORITY)	 ISR = ISR_RX_VAL;
-	else if (val & TX_PRIORITY)	 ISR = ISR_TX_VAL;
-	else if (val & MSR_PRIORITY) ISR = ISR_MSR_VAL;
+	if (val & ERROR_PRIORITY)			ISR = ISR_ERROR_VAL;
+	else if (val & TIMEOUT_PRIORITY)	ISR = ISR_FIFOTIMEOUT_VAL;
+	else if (val & RX_PRIORITY)			ISR = ISR_RX_VAL;
+	else if (val & TX_PRIORITY)			ISR = ISR_TX_VAL;
+	else if (val & MSR_PRIORITY)		ISR = ISR_MSR_VAL;
 	else ISR = ISR_CLEAR_VAL;
 
-	if(val && !irq_active) {
+	if(val && !irq_active) 
+	{
 		irq_active=true;
-		PIC_ActivateIRQ(irq);
-
+		if(op2) {
+			PIC_ActivateIRQ(irq);
 #if SERIAL_DEBUG
-		if(dbg_interrupt)
-			fprintf(debugfp,"%12.3f IRQ%d on.\r\n",PIC_FullIndex(),irq);
+			log_ser(dbg_interrupt,"IRQ%d on.",irq);
 #endif
-	}
-
-	if(!val && irq_active) {
+		}
+	} else if((!val) && irq_active) {
 		irq_active=false;
-		PIC_DeActivateIRQ(irq);
-
+		if(op2) { 
+			PIC_DeActivateIRQ(irq);
 #if SERIAL_DEBUG
-		if(dbg_interrupt)
-			fprintf(debugfp,"%12.3f IRQ%d off.\r\n",PIC_FullIndex(),irq);
+			log_ser(dbg_interrupt,"IRQ%d off.",irq);
 #endif
+		}
 	}
 }
 
@@ -330,108 +361,98 @@ void CSerial::ComputeInterrupts () {
 /* Can a byte be received?                                                  **/
 /*****************************************************************************/
 bool CSerial::CanReceiveByte() {
-	return (LSR & LSR_RX_DATA_READY_MASK) == 0;
+	return !rxfifo->isFull();
 }
 
 /*****************************************************************************/
 /* A byte was received                                                      **/
 /*****************************************************************************/
-void CSerial::receiveByte (Bit8u data) {
-
+void CSerial::receiveByteEx (Bit8u data, Bit8u error) {
 #if SERIAL_DEBUG
-	if(dbg_serialtraffic)
-		fprintf(debugfp,loopback_data<0x10? "%12.3f rx 0x%02x (%u)\r\n":
-											"%12.3f rx 0x%02x (%c)\r\n",
-			PIC_FullIndex(), data, data);
+	log_ser(dbg_serialtraffic,data<0x10 ? "\t\t\t\trx 0x%02x (%u)":
+		"\t\t\t\trx 0x%02x (%c)", data, data);
 #endif
+	if (!(rxfifo->addb(data))) {
+		// Overrun error ;o
+		error |= LSR_OVERRUN_ERROR_MASK;
+	}
+	removeEvent(SERIAL_RX_TIMEOUT_EVENT);
+	if(rxfifo->getUsage()==rx_interrupt_threshold) rise (RX_PRIORITY);
+	else setEvent(SERIAL_RX_TIMEOUT_EVENT,bytetime*4.0f);
 
-	if (LSR & LSR_RX_DATA_READY_MASK) {	// Overrun error ;o
+	if(error) {
+		// A lot of UART chips generate a framing error too when receiving break
+		if(error&LSR_RX_BREAK_MASK) error |= LSR_FRAMING_ERROR_MASK;
+#if SERIAL_DEBUG
+		log_ser(dbg_serialtraffic,"with error: framing=%d,overrun=%d,break=%d,parity=%d",
+			(error&LSR_FRAMING_ERROR_MASK)>0,(error&LSR_OVERRUN_ERROR_MASK)>0,
+			(error&LSR_RX_BREAK_MASK)>0,(error&LSR_PARITY_ERROR_MASK)>0);
+#endif
+		if(FCR&FCR_ACTIVATE) {
+			// error and FIFO active
+			if(!errorfifo->isFull()) {
+				errors_in_fifo++;
+				errorfifo->addb(error);
+			}
+			else {
+				Bit8u toperror=errorfifo->getTop();
+				if(!toperror) errors_in_fifo++;
+				errorfifo->addb(error|toperror);
+			}
+			if(errorfifo->probeByte()) {
+				// the next byte in the error fifo has an error
+				rise (ERROR_PRIORITY);
+				LSR |= error;
+			}
+		} else {
+			// error and FIFO inactive
+			rise (ERROR_PRIORITY);
+			LSR |= error;
+		};
+        if(error&LSR_PARITY_ERROR_MASK) {
+			parityErrors++;
+		};
+		if(error&LSR_OVERRUN_ERROR_MASK) {
+			overrunErrors++;
+			if(!GETFLAG(IF)) overrunIF0++;
+#if SERIAL_DEBUG
+			log_ser(dbg_serialtraffic,"rx overrun (IF=%d)", GETFLAG(IF)>0);
+#endif
+		};
+		if(error&LSR_FRAMING_ERROR_MASK) {
+			framingErrors++;
+		}
+		if(error&LSR_RX_BREAK_MASK) {
+			breakErrors++;
+		}
+		// trigger status window error notification
 		if(!errormsg_pending) {
 			errormsg_pending=true;
 			setEvent(SERIAL_ERRMSG_EVENT,1000);
 		}
-		overrunErrors++;
-		Bitu iflag= GETFLAG(IF);
-		if(!iflag)overrunIF0++;
-
-#if SERIAL_DEBUG
-		if(dbg_serialtraffic)
-			fprintf(debugfp, "%12.3f rx overrun (IF=%d)\r\n",
-				PIC_FullIndex(), iflag);
-#endif
-
-		LSR |= LSR_OVERRUN_ERROR_MASK;
-		rise (ERROR_PRIORITY);
 	} else {
-		RHR = data;
-		LSR |= LSR_RX_DATA_READY_MASK;
-		rise (RX_PRIORITY);
+		// no error
+		if(FCR&FCR_ACTIVATE) {
+			errorfifo->addb(error);
+		}
 	}
 }
 
-/*****************************************************************************/
-/* A line error was received                                                **/
-/*****************************************************************************/
-void CSerial::receiveError (Bit8u errorword) {
-	
-	if(!errormsg_pending) {
-			errormsg_pending=true;
-			setEvent(SERIAL_ERRMSG_EVENT,1000);
-	}
-	if(errorword&LSR_PARITY_ERROR_MASK) {
-		parityErrors++;
-
-#if SERIAL_DEBUG
-		if(dbg_serialtraffic)
-			fprintf(debugfp, "%12.3f parity error\r\n",
-				PIC_FullIndex());
-#endif
-
-	}
-	if(errorword&LSR_FRAMING_ERROR_MASK) {
-		framingErrors++;
-
-#if SERIAL_DEBUG
-		if(dbg_serialtraffic)
-			fprintf(debugfp, "%12.3f framing error\r\n",
-				PIC_FullIndex());
-#endif
-
-	}
-	if(errorword&LSR_RX_BREAK_MASK) {
-		breakErrors++;
-
-#if SERIAL_DEBUG
-		if(dbg_serialtraffic)
-			fprintf(debugfp, "%12.3f break received\r\n",
-				PIC_FullIndex());
-#endif
-
-	}
-	LSR |= errorword;
-
-	rise (ERROR_PRIORITY);
+void CSerial::receiveByte (Bit8u data) {
+	receiveByteEx(data,0);
 }
 
 /*****************************************************************************/
 /* ByteTransmitting: Byte has made it from THR to TX.                       **/
 /*****************************************************************************/
 void CSerial::ByteTransmitting() {
-	switch(LSR&(LSR_TX_HOLDING_EMPTY_MASK|LSR_TX_EMPTY_MASK))
-	{
-		case LSR_TX_HOLDING_EMPTY_MASK|LSR_TX_EMPTY_MASK:
-			// bad case there must have been one
-		case LSR_TX_HOLDING_EMPTY_MASK:
-		case LSR_TX_EMPTY_MASK: // holding full but workreg empty impossible
-			LOG_MSG("Internal error in serial port(1)(0x%x).",LSR);
-			break;
-		case 0: // THR is empty now.
-			LSR |= LSR_TX_HOLDING_EMPTY_MASK;
-			
-			// trigger interrupt
-			rise (TX_PRIORITY);
-			break;
-	}
+	if(sync_guardtime) {
+		//LOG_MSG("byte transmitting after guard");
+		//if(txfifo->isEmpty()) LOG_MSG("Serial port: FIFO empty when it should not");
+		sync_guardtime=false;
+		txfifo->getb();
+	} //else LOG_MSG("byte transmitting");
+	if(txfifo->isEmpty())rise (TX_PRIORITY);
 }
 
 
@@ -439,39 +460,23 @@ void CSerial::ByteTransmitting() {
 /* ByteTransmitted: When a byte was sent, notify here.                      **/
 /*****************************************************************************/
 void CSerial::ByteTransmitted () {
-	switch(LSR&(LSR_TX_HOLDING_EMPTY_MASK|LSR_TX_EMPTY_MASK))
-	{
-		case LSR_TX_HOLDING_EMPTY_MASK|LSR_TX_EMPTY_MASK:
-			// bad case there must have been one
-		case LSR_TX_EMPTY_MASK: // holding full but workreg empty impossible
-			LOG_MSG("Internal error in serial port(2).");
-			break;
-		
-		case LSR_TX_HOLDING_EMPTY_MASK: // now both are empty
-			LSR |= LSR_TX_EMPTY_MASK;
-			break;
+	if(!txfifo->isEmpty()) {
+		// there is more data
+		Bit8u data = txfifo->getb();
+#if SERIAL_DEBUG
+		log_ser(dbg_serialtraffic,data<0x10?
+			"\t\t\t\t\ttx 0x%02x (%u) (from buffer)":
+			"\t\t\t\t\ttx 0x%02x (%c) (from buffer)",data,data);
+#endif
+		if (loopback) setEvent(SERIAL_TX_LOOPBACK_EVENT, bytetime);
+		else transmitByte(data,false);
+		if(txfifo->isEmpty())rise (TX_PRIORITY);
 
-		case 0: // now one is empty, send the other one
-			LSR |= LSR_TX_HOLDING_EMPTY_MASK;
-			if (loopback) {
-				loopback_data=THR;
-				setEvent(SERIAL_TX_LOOPBACK_EVENT, bytetime);
-			}
-			else {
-
-		#if SERIAL_DEBUG
-				if(dbg_serialtraffic)
-					fprintf(debugfp,THR<0x10? "%12.3f tx 0x%02x (%u) (from THR)\r\n":
-														"%12.3f tx 0x%02x (%c) (from THR)\r\n",
-						PIC_FullIndex(),THR,
-						THR);
-		#endif
-
-				transmitByte(THR,false);
-			}
-			// It's ok here.
-			rise (TX_PRIORITY);
-			break;
+	} else {
+#if SERIAL_DEBUG
+		log_ser(dbg_serialtraffic,"tx buffer empty.");
+#endif
+		LSR |= LSR_TX_EMPTY_MASK;
 	}
 }
 
@@ -488,51 +493,43 @@ void CSerial::Write_THR (Bit8u data) {
 		changeLineProperties();
 	} else {
 		// write to THR
-		clear (TX_PRIORITY);
+        clear (TX_PRIORITY);
 
-		switch(LSR&(LSR_TX_HOLDING_EMPTY_MASK|LSR_TX_EMPTY_MASK))
-		{
-			case 0: // both full - overflow
-#if SERIAL_DEBUG
-				if(dbg_serialtraffic) fprintf(debugfp, "%12.3f tx overflow\r\n",
-					PIC_FullIndex());
-#endif
-				// overwrite THR
-				THR = data;
-				break;
-		
-			case LSR_TX_EMPTY_MASK: // holding full but workreg empty impossible
-				LOG_MSG("Internal error in serial port(3).");
-				break;
-		
-			case LSR_TX_HOLDING_EMPTY_MASK: // now both are full
-				LSR &= (~LSR_TX_HOLDING_EMPTY_MASK);
-				THR = data;
-				break;
-
-			case LSR_TX_HOLDING_EMPTY_MASK|LSR_TX_EMPTY_MASK:
-				// both are full until the first delay has passed
-				THR=data;
-				LSR &= (~LSR_TX_EMPTY_MASK);
-				LSR &= (~LSR_TX_HOLDING_EMPTY_MASK);
-				if(loopback) setEvent(SERIAL_THR_LOOPBACK_EVENT, bytetime/10);
-				else {
+		if((LSR & LSR_TX_EMPTY_MASK))
+		{	// we were idle before
+			//LOG_MSG("starting new transmit cycle");
+			//if(sync_guardtime) LOG_MSG("Serial port internal error 1");
+			//if(!(LSR & LSR_TX_EMPTY_MASK)) LOG_MSG("Serial port internal error 2");
+			//if(txfifo->getUsage()) LOG_MSG("Serial port internal error 3");
 			
+			// need "warming up" time
+			sync_guardtime=true;
+			// block the fifo so it returns THR full (or not in case of FIFO on)
+			txfifo->addb(data); 
+			// transmit shift register is busy
+			LSR &= (~LSR_TX_EMPTY_MASK);
+			if(loopback) setEvent(SERIAL_THR_LOOPBACK_EVENT, bytetime/10);
+			else {
 #if SERIAL_DEBUG
-					if(dbg_serialtraffic)
-						fprintf(debugfp,data<0x10? "%12.3f tx 0x%02x (%u)\r\n":
-												   "%12.3f tx 0x%02x (%c)\r\n",
-							PIC_FullIndex(),data,
-							data);
-#endif	
-							
-					transmitByte (data,true);
+				log_ser(dbg_serialtraffic,data<0x10?
+					"\t\t\t\t\ttx 0x%02x (%u) [FIFO=%2d]":
+					"\t\t\t\t\ttx 0x%02x (%c) [FIFO=%2d]",data,data,txfifo->getUsage());
+#endif
+				transmitByte (data,true);
+			}
+		} else {
+			//  shift register is transmitting
+			if(!txfifo->addb(data)) {
+				// TX overflow
+#if SERIAL_DEBUG
+				log_ser(dbg_serialtraffic,"tx overflow");
+#endif
+				txOverrunErrors++;
+				if(!errormsg_pending) {
+					errormsg_pending=true;
+					setEvent(SERIAL_ERRMSG_EVENT,1000);
 				}
-				
-				// triggered
-				// when holding gets empty
-				// rise (TX_PRIORITY);
-				break;
+			}
 		}
 	}
 }
@@ -545,9 +542,26 @@ Bitu CSerial::Read_RHR () {
 	// 0-7 received data
 	if ((LCR & LCR_DIVISOR_Enable_MASK)) return baud_divider&0xff;
 	else {
-		clear (RX_PRIORITY);
-		LSR &= (~LSR_RX_DATA_READY_MASK);
-		return RHR;
+		Bit8u data=rxfifo->getb();
+		if(FCR&FCR_ACTIVATE) {
+			Bit8u error=errorfifo->getb();
+			if(error) errors_in_fifo--;
+			// new error
+			if(!rxfifo->isEmpty()) {
+				error=errorfifo->probeByte();
+				if(error) {
+					LSR |= error;
+					rise(ERROR_PRIORITY);
+				}
+			}
+		}
+		// Reading RHR resets the FIFO timeout
+		clear (TIMEOUT_PRIORITY);
+		// RX int. is cleared if the buffer holds less data than the threshold
+		if(rxfifo->getUsage()<rx_interrupt_threshold)clear(RX_PRIORITY);
+		removeEvent(SERIAL_RX_TIMEOUT_EVENT);
+		if(!rxfifo->isEmpty()) setEvent(SERIAL_RX_TIMEOUT_EVENT,bytetime*4.0f);
+		return data;
 	}
 }
 
@@ -564,7 +578,7 @@ Bitu CSerial::Read_IER () {
 	// 4-7	0
 
 	if (LCR & LCR_DIVISOR_Enable_MASK) return baud_divider>>8;
-	else return IER;
+	else return IER&0x0f;
 }
 
 void CSerial::Write_IER (Bit8u data) {
@@ -573,11 +587,12 @@ void CSerial::Write_IER (Bit8u data) {
 		baud_divider |= ((Bit16u)data)<<8;
 		changeLineProperties();
 	} else {
-
-		IER = data&0xF;
-		if ((LSR & LSR_TX_HOLDING_EMPTY_MASK) && (IER&TX_PRIORITY))
+		// Retrigger TX interrupt
+		if (txfifo->isEmpty()&& (data&TX_PRIORITY))
 			waiting_interrupts |= TX_PRIORITY;
 		
+		IER = data&0xF;
+		if((FCR&FCR_ACTIVATE)&&data&RX_PRIORITY) IER |= TIMEOUT_PRIORITY; 
 		ComputeInterrupts();
 	}
 }
@@ -593,6 +608,7 @@ Bitu CSerial::Read_ISR () {
 	// 1-3	identification
 	//      011 LSR
 	//		010 RXRDY
+	//		110 RX_TIMEOUT
 	//		001 TXRDY
 	//		000 MSR
 	// 4-7	0
@@ -601,14 +617,44 @@ Bitu CSerial::Read_ISR () {
 	Bit8u retval = ISR;
 
 	// clear changes ISR!! mean..
-	if(ISR==ISR_TX_VAL) clear (TX_PRIORITY);
+	if(ISR==ISR_TX_VAL) clear(TX_PRIORITY);
+	if(FCR&FCR_ACTIVATE) retval |= FIFO_STATUS_ACTIVE;
+
 	return retval;
 }
 
+#define BIT_CHANGE_H(oldv,newv,bitmask) (!(oldv&bitmask) && (newv&bitmask))
+#define BIT_CHANGE_L(oldv,newv,bitmask) ((oldv&bitmask) && !(newv&bitmask))
+
 void CSerial::Write_FCR (Bit8u data) {
-	if((!fifo_warn) && (data&0x1)) {
-		fifo_warn=true;
-		LOG_MSG("Serial%d: Warning: Tried to activate FIFO.",COMNUMBER);
+	if(BIT_CHANGE_H(FCR,data,FCR_ACTIVATE)) {
+		// FIFO was switched on
+		errors_in_fifo=0; // should already be 0
+		errorfifo->setSize(fifosize);
+		rxfifo->setSize(fifosize);
+		txfifo->setSize(fifosize);
+	} else if(BIT_CHANGE_L(FCR,data,FCR_ACTIVATE)) {
+		// FIFO was switched off
+		errors_in_fifo=0;
+		errorfifo->setSize(1);
+		rxfifo->setSize(1);
+		txfifo->setSize(1);
+		rx_interrupt_threshold=1;
+	}
+	FCR=data&0xCF;
+	if(FCR&FCR_CLEAR_RX) {
+		errors_in_fifo=0;
+		errorfifo->clear();
+		rxfifo->clear();
+	}
+	if(FCR&FCR_CLEAR_TX) txfifo->clear();
+	if(FCR&FCR_ACTIVATE) {
+		switch(FCR>>6) {
+			case 0: rx_interrupt_threshold=1; break;
+			case 1: rx_interrupt_threshold=4; break;
+			case 2: rx_interrupt_threshold=8; break;
+			case 3: rx_interrupt_threshold=14; break;
+		}
 	}
 }
 
@@ -643,10 +689,8 @@ void CSerial::Write_LCR (Bit8u data) {
 			// TODO: set loopback break event to reveiveError after
 		}
 #if SERIAL_DEBUG
-		if(dbg_serialtraffic)
-			fprintf(debugfp,((LCR & LCR_BREAK_MASK)!=0)?
-						"%12.3f break on.\r\n":
-						"%12.3f break off.\r\n", PIC_FullIndex());
+		log_ser(dbg_serialtraffic,((LCR & LCR_BREAK_MASK)!=0) ?
+			"break on.":"break off.");
 #endif
 	}
 }
@@ -675,7 +719,7 @@ Bitu CSerial::Read_MCR () {
 
 void CSerial::Write_MCR (Bit8u data) {
 	// WARNING: At the time setRTSDTR is called rts and dsr members are still wrong.
-
+	if(data&FIFO_FLOWCONTROL) LOG_MSG("Warning: tried to activate hardware handshake.");
 	bool temp_dtr = data & MCR_DTR_MASK? true:false;
 	bool temp_rts = data & MCR_RTS_MASK? true:false;
 	bool temp_op1 = data & MCR_OP1_MASK? true:false;
@@ -718,34 +762,34 @@ void CSerial::Write_MCR (Bit8u data) {
 				// both difference
 
 #if SERIAL_DEBUG
-			if(dbg_modemcontrol)
-			{
-				fprintf(debugfp,temp_rts?"%12.3f RTS on.\r\n":
-									"%12.3f RTS off.\r\n", PIC_FullIndex());
-				fprintf(debugfp,temp_dtr?"%12.3f DTR on.\r\n":
-									"%12.3f DTR off.\r\n", PIC_FullIndex());
-			}
+				log_ser(dbg_modemcontrol,"RTS %x.",temp_rts);
+				log_ser(dbg_modemcontrol,"DTR %x.",temp_dtr);
 #endif
 				setRTSDTR(temp_rts, temp_dtr);
 			} else {
 				// only RTS
 
 #if SERIAL_DEBUG
-			if(dbg_modemcontrol)
-				fprintf(debugfp,temp_rts?"%12.3f RTS on.\r\n":"%12.3f RTS off.\r\n", PIC_FullIndex());
+				log_ser(dbg_modemcontrol,"RTS %x.",temp_rts);
 #endif
-
 				setRTS(temp_rts);
 			}
 		} else if(temp_dtr!=dtr) {
 			// only DTR
 #if SERIAL_DEBUG
-			if(dbg_modemcontrol)
-				fprintf(debugfp,temp_dtr?"%12.3f DTR on.\r\n":"%12.3f DTR off.\r\n", PIC_FullIndex());
+				log_ser(dbg_modemcontrol,"%DTR %x.",temp_dtr);
 #endif
 			setDTR(temp_dtr);
 		}
 	}
+	// interrupt logic: if OP2 is 0, the IRQ line is tristated (pulled high)
+	if((!op2) && temp_op2) {
+		// irq has been enabled (tristate high -> irq level)
+		if(!irq_active) PIC_DeActivateIRQ(irq);
+	} else if(op2 && (!temp_op2)) {
+		if(!irq_active) PIC_ActivateIRQ(irq); 
+	}
+
 	dtr=temp_dtr;
 	rts=temp_rts;
 	op1=temp_op1;
@@ -761,7 +805,10 @@ void CSerial::Write_MCR (Bit8u data) {
 // - event from real serial port
 // - loopback
 Bitu CSerial::Read_LSR () {
-	Bitu retval = LSR;
+	Bitu retval = LSR & (LSR_ERROR_MASK|LSR_TX_EMPTY_MASK);
+	if(txfifo->isEmpty()) retval |= LSR_TX_HOLDING_EMPTY_MASK;
+	if(!(rxfifo->isEmpty()))retval |= LSR_RX_DATA_READY_MASK;
+	if(errors_in_fifo) retval |= FIFO_ERROR;
 	LSR &= (~LSR_ERROR_MASK);			// clear error bits on read
 	clear (ERROR_PRIORITY);
 	return retval;
@@ -872,8 +919,7 @@ void CSerial::setRI (bool value) {
 	if (value != ri) {
 
 #if SERIAL_DEBUG
-		if(dbg_modemcontrol)
-			fprintf(debugfp,value?"%12.3f RI  on.\r\n":"%12.3f RI  off.\r\n", PIC_FullIndex());
+		log_ser(dbg_modemcontrol,"%RI  %x.",value);
 #endif
 		// don't change delta when in loopback mode
 		ri=value;
@@ -887,8 +933,7 @@ void CSerial::setRI (bool value) {
 void CSerial::setDSR (bool value) {
 	if (value != dsr) {
 #if SERIAL_DEBUG
-		if(dbg_modemcontrol)
-			fprintf(debugfp,value?"%12.3f DSR on.\r\n":"%12.3f DSR off.\r\n", PIC_FullIndex());
+		log_ser(dbg_modemcontrol,"DSR %x.",value);
 #endif
 		// don't change delta when in loopback mode
 		dsr=value;
@@ -902,8 +947,7 @@ void CSerial::setDSR (bool value) {
 void CSerial::setCD (bool value) {
 	if (value != cd) {
 #if SERIAL_DEBUG
-		if(dbg_modemcontrol)
-			fprintf(debugfp,value?"%12.3f CD  on.\r\n":"%12.3f CD  off.\r\n", PIC_FullIndex());
+		log_ser(dbg_modemcontrol,"CD  %x.",value);
 #endif
 		// don't change delta when in loopback mode
 		cd=value;
@@ -917,8 +961,7 @@ void CSerial::setCD (bool value) {
 void CSerial::setCTS (bool value) {
 	if (value != cts) {
 #if SERIAL_DEBUG
-		if(dbg_modemcontrol)
-			fprintf(debugfp,value?"%12.3f CTS on.\r\n":"%12.3f CTS off.\r\n", PIC_FullIndex());
+		log_ser(dbg_modemcontrol,"CTS %x.",value);
 #endif
 		// don't change delta when in loopback mode
 		cts=value;
@@ -945,8 +988,6 @@ void CSerial::Init_Registers () {
 	Bit8u lcrresult = 0;
 	Bit16u baudresult = 0;
 
-	RHR = 0;
-	THR = 0;
 	IER = 0;
 	ISR = 0x1;
 	LCR = 0;
@@ -957,6 +998,10 @@ void CSerial::Init_Registers () {
 	op1=true;
 	op2=true;
 
+	sync_guardtime=false;
+	FCR=0xff;
+	Write_FCR(0x00);
+
 
 	LSR = 0x60;
 	d_cts = true;	
@@ -1020,9 +1065,16 @@ void CSerial::Init_Registers () {
 	Write_LCR (lcrresult);
 	updateMSR();
 	Read_MSR();
+	PIC_DeActivateIRQ(irq);
 }
 
 CSerial::CSerial(Bitu id, CommandLine* cmd) {
+	idnumber=id;
+	Bit16u base = serial_baseaddr[id];
+
+	irq = serial_defaultirq[id];
+	getBituSubstring("irq:",&irq, cmd);
+	if (irq < 2 || irq > 15) irq = serial_defaultirq[id];
 
 #if SERIAL_DEBUG
 	dbg_serialtraffic = cmd->FindExist("dbgtr", false);
@@ -1030,44 +1082,55 @@ CSerial::CSerial(Bitu id, CommandLine* cmd) {
 	dbg_register      = cmd->FindExist("dbgreg", false);
 	dbg_interrupt     = cmd->FindExist("dbgirq", false);
 	dbg_aux			  = cmd->FindExist("dbgaux", false);
+	
+	if(cmd->FindExist("dbgall", false)) {
+		dbg_serialtraffic= 
+		dbg_modemcontrol= 
+		dbg_register=
+		dbg_interrupt=
+		dbg_aux= true;
+	}
 
 
 	if(dbg_serialtraffic|dbg_modemcontrol|dbg_register|dbg_interrupt|dbg_aux)
 		debugfp=OpenCaptureFile("serlog",".serlog.txt");
 	else debugfp=0;
 
+	if(debugfp == 0) {
+		dbg_serialtraffic= 
+		dbg_modemcontrol= 
+		dbg_register=
+		dbg_interrupt=
+		dbg_aux= false;
+	} else {
+		std::string cleft;
+		cmd->GetStringRemain(cleft);
+
+		log_ser(true,"Serial%d: BASE %3x, IRQ %d, initstring \"%s\"\r\n\r\n",
+			COMNUMBER,base,irq,cleft.c_str());
+	}
 #endif
+	fifosize=16;
 
+	errorfifo = new MyFifo(fifosize);
+	rxfifo = new MyFifo(fifosize);
+	txfifo = new MyFifo(fifosize);
 
-
-	idnumber=id;
 	mydosdevice=new device_COM(this);
 	DOS_AddDevice(mydosdevice);
-	Bit16u base = serial_baseaddr[id];
-	fifo_warn=false;
 
 	errormsg_pending=false;
 	framingErrors=0;
 	parityErrors=0;
 	overrunErrors=0;
+	txOverrunErrors=0;
 	overrunIF0=0;
 	breakErrors=0;
 	
-	// find the IRQ
-	irq = serial_defaultirq[id];
-	getBituSubstring("irq:",&irq, cmd);
-	if (irq < 2 || irq > 15) irq = serial_defaultirq[id];
-
-
 	for (Bitu i = 0; i <= 7; i++) {
 		WriteHandler[i].Install (i + base, SERIAL_Write, IO_MB);
 		ReadHandler[i].Install (i + base, SERIAL_Read, IO_MB);
 	}
-
-#if SERIAL_DEBUG
-	if(debugfp) fprintf(debugfp,"COM%d: BASE %3x, IRQ %d\r\n\r\n",
-		COMNUMBER,base,irq);
-#endif
 }
 
 bool CSerial::getBituSubstring(const char* name,Bitu* data, CommandLine* cmd) {
@@ -1082,19 +1145,16 @@ CSerial::~CSerial(void) {
 	DOS_DelDevice(mydosdevice);
 	for(Bitu i = 0; i <= SERIAL_BASE_EVENT_COUNT; i++)
 		removeEvent(i);
-}
+};
 bool CSerial::Getchar(Bit8u* data, Bit8u* lsr, bool wait_dsr, Bitu timeout) {
-	
 	double starttime=PIC_FullIndex();
-	// wait for it to become empty
 	// wait for DSR on
 	if(wait_dsr) {
 		while((!(Read_MSR()&0x20))&&(starttime>PIC_FullIndex()-timeout))
 			CALLBACK_Idle();
 		if(!(starttime>PIC_FullIndex()-timeout)) {
-			#if SERIAL_DEBUG
-if(dbg_aux)
-		fprintf(debugfp,"%12.3f Getchar status timeout: MSR 0x%x\r\n", PIC_FullIndex(),Read_MSR());
+#if SERIAL_DEBUG
+			log_ser(dbg_aux,"Getchar status timeout: MSR 0x%x",Read_MSR());
 #endif
 			return false;
 		}
@@ -1104,21 +1164,16 @@ if(dbg_aux)
 		CALLBACK_Idle();
 	
 	if(!(starttime>PIC_FullIndex()-timeout)) {
-		#if SERIAL_DEBUG
-if(dbg_aux)
-		fprintf(debugfp,"%12.3f Getchar data timeout: MSR 0x%x\r\n", PIC_FullIndex(),Read_MSR());
+#if SERIAL_DEBUG
+		log_ser(dbg_aux,"Getchar data timeout: MSR 0x%x",Read_MSR());
 #endif
 		return false;
 	}
-
-
 	*data=Read_RHR();
 
 #if SERIAL_DEBUG
-	if(dbg_aux)
-		fprintf(debugfp,"%12.3f API read success: 0x%x\r\n", PIC_FullIndex(),*data);
+	log_ser(dbg_aux,"Getchar read 0x%x",*data);
 #endif
-
 	return true;
 }
 
@@ -1126,9 +1181,8 @@ if(dbg_aux)
 bool CSerial::Putchar(Bit8u data, bool wait_dsr, bool wait_cts, Bitu timeout) {
 	
 	double starttime=PIC_FullIndex();
-	//Bit16u starttime=
 	// wait for it to become empty
-	while(!(LSR&0x20)) {
+	while(!(Read_LSR()&0x20)) {
 		CALLBACK_Idle();
 	}
 	// wait for DSR+CTS on
@@ -1142,13 +1196,10 @@ bool CSerial::Putchar(Bit8u data, bool wait_dsr, bool wait_cts, Bitu timeout) {
 		} else if(wait_cts) {
 			while(!(Read_MSR()&0x10)&&(starttime>PIC_FullIndex()-timeout))
 				CALLBACK_Idle();
-
 		} 
 		if(!(starttime>PIC_FullIndex()-timeout)) {
 #if SERIAL_DEBUG
-			if(dbg_aux)
-				fprintf(debugfp,"%12.3f Putchar timeout: MSR 0x%x\r\n",
-					PIC_FullIndex(),Read_MSR());
+			log_ser(dbg_aux,"Putchar timeout: MSR 0x%x",Read_MSR());
 #endif
 			return false;
 		}
@@ -1156,8 +1207,7 @@ bool CSerial::Putchar(Bit8u data, bool wait_dsr, bool wait_cts, Bitu timeout) {
 	Write_THR(data);
 
 #if SERIAL_DEBUG
-	if(dbg_aux)
-		fprintf(debugfp,"%12.3f API write success: 0x%x\r\n", PIC_FullIndex(),data);
+	log_ser(dbg_aux,"Putchar 0x%x",data);
 #endif
 
 	return true;
