@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2009  The DOSBox Team
+ *  Copyright (C) 2002-2010  The DOSBox Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-/* $Id: vga_other.cpp,v 1.25 2009/01/25 12:00:51 c2woody Exp $ */
+/* $Id: vga_other.cpp,v 1.28 2009-07-11 10:25:24 c2woody Exp $ */
 
 #include <string.h>
 #include <math.h>
@@ -28,57 +28,60 @@
 #include "render.h"
 #include "mapper.h"
 
-static void write_crtc_index_other(Bitu port,Bitu val,Bitu iolen) {
-	vga.other.index=val;
+static void write_crtc_index_other(Bitu /*port*/,Bitu val,Bitu /*iolen*/) {
+	vga.other.index=(Bit8u)val;
 }
 
-static Bitu read_crtc_index_other(Bitu port,Bitu iolen) {
+static Bitu read_crtc_index_other(Bitu /*port*/,Bitu /*iolen*/) {
 	return vga.other.index;
 }
 
-static void write_crtc_data_other(Bitu port,Bitu val,Bitu iolen) {
+static void write_crtc_data_other(Bitu /*port*/,Bitu val,Bitu /*iolen*/) {
 	switch (vga.other.index) {
 	case 0x00:		//Horizontal total
 		if (vga.other.htotal ^ val) VGA_StartResize();
-		vga.other.htotal=val;
+		vga.other.htotal=(Bit8u)val;
 		break;
 	case 0x01:		//Horizontal displayed chars
 		if (vga.other.hdend ^ val) VGA_StartResize();
-		vga.other.hdend=val;
+		vga.other.hdend=(Bit8u)val;
 		break;
 	case 0x02:		//Horizontal sync position
-		vga.other.hsyncp=val;
+		vga.other.hsyncp=(Bit8u)val;
 		break;
-	case 0x03:		//Horizontal and vertical sync width
-		vga.other.syncw=val;
+	case 0x03:		//Horizontal sync width
+		if (machine==MCH_TANDY) vga.other.vsyncw=(Bit8u)(val >> 4);
+		else vga.other.vsyncw = 16; // The MC6845 has a fixed v-sync width of 16 lines
+		vga.other.hsyncw=(Bit8u)(val & 0xf);
 		break;
 	case 0x04:		//Vertical total
 		if (vga.other.vtotal ^ val) VGA_StartResize();
-		vga.other.vtotal=val;
+		vga.other.vtotal=(Bit8u)val;
 		break;
 	case 0x05:		//Vertical display adjust
 		if (vga.other.vadjust ^ val) VGA_StartResize();
-		vga.other.vadjust=val;
+		vga.other.vadjust=(Bit8u)val;
 		break;
 	case 0x06:		//Vertical rows
 		if (vga.other.vdend ^ val) VGA_StartResize();
-		vga.other.vdend=val;
+		vga.other.vdend=(Bit8u)val;
 		break;
 	case 0x07:		//Vertical sync position
-		vga.other.vsyncp=val;
+		vga.other.vsyncp=(Bit8u)val;
 		break;
 	case 0x09:		//Max scanline
-		if (vga.other.max_scanline ^ val) VGA_StartResize();
-		vga.other.max_scanline=val;
+		val &= 0x1f; // VGADOC says bit 0-3 but the MC6845 datasheet says bit 0-4
+ 		if (vga.other.max_scanline ^ val) VGA_StartResize();
+		vga.other.max_scanline=(Bit8u)val;
 		break;
 	case 0x0A:	/* Cursor Start Register */
-		vga.other.cursor_start = val & 0x3f;
-		vga.draw.cursor.sline = val&0x1f;
+		vga.other.cursor_start = (Bit8u)(val & 0x3f);
+		vga.draw.cursor.sline = (Bit8u)(val&0x1f);
 		vga.draw.cursor.enabled = ((val & 0x60) != 0x20);
 		break;
 	case 0x0B:	/* Cursor End Register */
-		vga.other.cursor_end = val&0x1f;
-		vga.draw.cursor.eline = val&0x1f;
+		vga.other.cursor_end = (Bit8u)(val&0x1f);
+		vga.draw.cursor.eline = (Bit8u)(val&0x1f);
 		break;
 	case 0x0C:	/* Start Address High Register */
 		vga.config.display_start=(vga.config.display_start & 0x00FF) | (val << 8);
@@ -88,23 +91,25 @@ static void write_crtc_data_other(Bitu port,Bitu val,Bitu iolen) {
 		break;
 	case 0x0E:	/*Cursor Location High Register */
 		vga.config.cursor_start&=0x00ff;
-		vga.config.cursor_start|=val << 8;
+		vga.config.cursor_start|=((Bit8u)val) << 8;
 		break;
 	case 0x0F:	/* Cursor Location Low Register */
 		vga.config.cursor_start&=0xff00;
-		vga.config.cursor_start|=val;
+		vga.config.cursor_start|=(Bit8u)val;
 		break;
 	case 0x10:	/* Light Pen High */
-		vga.other.lpen_high = val & 0x1f;		//only 6 bits
+		vga.other.lightpen &= 0xff;
+		vga.other.lightpen |= (val & 0x3f)<<8;		// only 6 bits
 		break;
 	case 0x11:	/* Light Pen Low */
-		vga.other.lpen_low = val;
+		vga.other.lightpen &= 0xff00;
+		vga.other.lightpen |= (Bit8u)val;
 		break;
 	default:
 		LOG(LOG_VGAMISC,LOG_NORMAL)("MC6845:Write %X to illegal index %x",val,vga.other.index);
 	}
 }
-static Bitu read_crtc_data_other(Bitu port,Bitu iolen) {
+static Bitu read_crtc_data_other(Bitu /*port*/,Bitu /*iolen*/) {
 	switch (vga.other.index) {
 	case 0x00:		//Horizontal total
 		return vga.other.htotal;
@@ -113,7 +118,9 @@ static Bitu read_crtc_data_other(Bitu port,Bitu iolen) {
 	case 0x02:		//Horizontal sync position
 		return vga.other.hsyncp;
 	case 0x03:		//Horizontal and vertical sync width
-		return vga.other.syncw;
+		if (machine==MCH_TANDY)
+			return vga.other.hsyncw | (vga.other.vsyncw << 4);
+		else return vga.other.hsyncw;
 	case 0x04:		//Vertical total
 		return vga.other.vtotal;
 	case 0x05:		//Vertical display adjust
@@ -129,26 +136,27 @@ static Bitu read_crtc_data_other(Bitu port,Bitu iolen) {
 	case 0x0B:	/* Cursor End Register */
 		return vga.other.cursor_end;
 	case 0x0C:	/* Start Address High Register */
-		return vga.config.display_start >> 8;
+		return (Bit8u)(vga.config.display_start >> 8);
 	case 0x0D:	/* Start Address Low Register */
-		return vga.config.display_start;
+		return (Bit8u)(vga.config.display_start & 0xff);
 	case 0x0E:	/*Cursor Location High Register */
-		return vga.config.cursor_start>>8;
+		return (Bit8u)(vga.config.cursor_start >> 8);
 	case 0x0F:	/* Cursor Location Low Register */
-		return vga.config.cursor_start;
+		return (Bit8u)(vga.config.cursor_start & 0xff);
 	case 0x10:	/* Light Pen High */
-		return vga.other.lpen_high;
+		return (Bit8u)(vga.other.lightpen >> 8);
 	case 0x11:	/* Light Pen Low */
-		return vga.other.lpen_low;
+		return (Bit8u)(vga.other.lightpen & 0xff);
 	default:
 		LOG(LOG_VGAMISC,LOG_NORMAL)("MC6845:Read from illegal index %x",vga.other.index);
 	}
-	return ~0;
+	return (Bitu)(~0);
 }
 
 static double hue_offset = 0.0;
 static Bit8u cga16_val = 0;
 static void update_cga16_color(void);
+static Bit8u herc_pal = 0;
 
 static void cga16_color_select(Bit8u val) {
 	cga16_val = val;
@@ -206,7 +214,7 @@ static void update_cga16_color(void) {
 			G = Y - 0.272*I - 0.647*Q; if (G < 0.0) G = 0.0; if (G > 1.0) G = 1.0;
 			B = Y - 1.105*I + 1.702*Q; if (B < 0.0) B = 0.0; if (B > 1.0) B = 1.0;
 
-			RENDER_SetPal(index,static_cast<Bit8u>(R*baseR),static_cast<Bit8u>(G*baseG),static_cast<Bit8u>(B*baseB));
+			RENDER_SetPal((Bit8u)index,static_cast<Bit8u>(R*baseR),static_cast<Bit8u>(G*baseG),static_cast<Bit8u>(B*baseB));
 		}
 	}
 }
@@ -275,17 +283,21 @@ static void TANDY_FindMode(void) {
 	}
 }
 
+void VGA_SetModeNow(VGAModes mode);
+
 static void PCJr_FindMode(void) {
 	if (vga.tandy.mode_control & 0x2) {
 		if (vga.tandy.mode_control & 0x10) {
 			/* bit4 of mode control 1 signals 16 colour graphics mode */
-			VGA_SetMode(M_TANDY16);
+			if (vga.mode==M_TANDY4) VGA_SetModeNow(M_TANDY16); // TODO lowres mode only
+			else VGA_SetMode(M_TANDY16);
 		} else if (vga.tandy.gfx_control & 0x08) {
 			/* bit3 of mode control 2 signals 2 colour graphics mode */
 			VGA_SetMode(M_TANDY2);
 		} else {
 			/* otherwise some 4-colour graphics mode */
-			VGA_SetMode(M_TANDY4);
+			if (vga.mode==M_TANDY16) VGA_SetModeNow(M_TANDY4);
+			else VGA_SetMode(M_TANDY4);
 		}
 		write_color_select(vga.tandy.color_select);
 	} else {
@@ -303,7 +315,7 @@ static void TandyCheckLineMask(void ) {
 		vga.tandy.line_shift = 13;
 		vga.tandy.addr_mask = (1 << 13) - 1;
 	} else {
-		vga.tandy.addr_mask = ~0;
+		vga.tandy.addr_mask = (Bitu)(~0);
 		vga.tandy.line_shift = 0;
 	}
 }
@@ -315,6 +327,7 @@ static void write_tandy_reg(Bit8u val) {
 			vga.tandy.mode_control=val;
 			VGA_SetBlinking(val & 0x20);
 			PCJr_FindMode();
+			vga.attr.disabled = (val&0x8)? 0: 1;
 		} else {
 			LOG(LOG_VGAMISC,LOG_NORMAL)("Unhandled Write %2X to tandy reg %X",val,vga.tandy.reg_index);
 		}
@@ -352,10 +365,11 @@ static void write_tandy_reg(Bit8u val) {
 	}
 }
 
-static void write_cga(Bitu port,Bitu val,Bitu iolen) {
+static void write_cga(Bitu port,Bitu val,Bitu /*iolen*/) {
 	switch (port) {
 	case 0x3d8:
-		vga.tandy.mode_control=val;
+		vga.tandy.mode_control=(Bit8u)val;
+		vga.attr.disabled = (val&0x8)? 0: 1; 
 		if (vga.tandy.mode_control & 0x2) {
 			if (vga.tandy.mode_control & 0x10) {
 				if (!(val & 0x4) && machine==MCH_CGA) {
@@ -371,36 +385,48 @@ static void write_cga(Bitu port,Bitu val,Bitu iolen) {
 		VGA_SetBlinking(val & 0x20);
 		break;
 	case 0x3d9:
-		write_color_select(val);
+		write_color_select((Bit8u)val);
 		break;
 	}
 }
 
-static void write_tandy(Bitu port,Bitu val,Bitu iolen) {
+static void write_tandy(Bitu port,Bitu val,Bitu /*iolen*/) {
 	switch (port) {
 	case 0x3d8:
-		vga.tandy.mode_control=val;
+		vga.tandy.mode_control=(Bit8u)val;
 		TandyCheckLineMask();
 		VGA_SetBlinking(val & 0x20);
 		TANDY_FindMode();
 		break;
 	case 0x3d9:
-		write_color_select(val);
+		write_color_select((Bit8u)val);
 		break;
 	case 0x3da:
-		vga.tandy.reg_index=val;
+		vga.tandy.reg_index=(Bit8u)val;
 		break;
-//	case 0x3db:	//Clear lightpen latch
+	case 0x3db:	// Clear lightpen latch
+		vga.other.lightpen_triggered = false;
 		break;
-//	case 0x3dc:	//Preset lightpen latch
+	case 0x3dc:	// Preset lightpen latch
+		if (!vga.other.lightpen_triggered) {
+			vga.other.lightpen_triggered = true; // TODO: this shows at port 3ba/3da bit 1
+			
+			double timeInFrame = PIC_FullIndex()-vga.draw.delay.framestart;
+			double timeInLine = fmod(timeInFrame,vga.draw.delay.htotal);
+			Bitu current_scanline = (Bitu)(timeInFrame / vga.draw.delay.htotal);
+			
+			vga.other.lightpen = (Bit16u)((vga.draw.address_add/2) * (current_scanline/2));
+			vga.other.lightpen += (Bit16u)((timeInLine / vga.draw.delay.hdend) *
+				((float)(vga.draw.address_add/2)));
+		}
 		break;
 //	case 0x3dd:	//Extended ram page address register:
 		break;
 	case 0x3de:
-		write_tandy_reg(val);
+		write_tandy_reg((Bit8u)val);
 		break;
 	case 0x3df:
-		vga.tandy.line_mask = val >> 6;
+		vga.tandy.line_mask = (Bit8u)(val >> 6);
 		vga.tandy.draw_bank = val & ((vga.tandy.line_mask&2) ? 0x6 : 0x7);
 		vga.tandy.mem_bank = (val >> 3) & ((vga.tandy.line_mask&2) ? 0x6 : 0x7);
 		TandyCheckLineMask();
@@ -409,18 +435,18 @@ static void write_tandy(Bitu port,Bitu val,Bitu iolen) {
 	}
 }
 
-static void write_pcjr(Bitu port,Bitu val,Bitu iolen) {
+static void write_pcjr(Bitu port,Bitu val,Bitu /*iolen*/) {
 	switch (port) {
 	case 0x3d9:
-		write_color_select(val);
+		write_color_select((Bit8u)val);
 		break;
 	case 0x3da:
-		if (vga.tandy.pcjr_flipflop) write_tandy_reg(val);
-		else vga.tandy.reg_index=val;
+		if (vga.tandy.pcjr_flipflop) write_tandy_reg((Bit8u)val);
+		else vga.tandy.reg_index=(Bit8u)val;
 		vga.tandy.pcjr_flipflop=!vga.tandy.pcjr_flipflop;
 		break;
 	case 0x3df:
-		vga.tandy.line_mask = val >> 6;
+		vga.tandy.line_mask = (Bit8u)(val >> 6);
 		vga.tandy.draw_bank = val & ((vga.tandy.line_mask&2) ? 0x6 : 0x7);
 		vga.tandy.mem_bank = (val >> 3) & ((vga.tandy.line_mask&2) ? 0x6 : 0x7);
 		vga.tandy.draw_base = &MemBase[vga.tandy.draw_bank * 16 * 1024];
@@ -431,7 +457,31 @@ static void write_pcjr(Bitu port,Bitu val,Bitu iolen) {
 	}
 }
 
-static void write_hercules(Bitu port,Bitu val,Bitu iolen) {
+static void CycleHercPal(bool pressed) {
+	if (!pressed) return;
+	if (++herc_pal>2) herc_pal=0;
+	Herc_Palette();
+	VGA_DAC_CombineColor(1,7);
+}
+	
+void Herc_Palette(void) {	
+	switch (herc_pal) {
+	case 0:	// White
+		VGA_DAC_SetEntry(0x7,0x2a,0x2a,0x2a);
+		VGA_DAC_SetEntry(0xf,0x3f,0x3f,0x3f);
+		break;
+	case 1:	// Amber
+		VGA_DAC_SetEntry(0x7,0x34,0x20,0x00);
+		VGA_DAC_SetEntry(0xf,0x3f,0x34,0x00);
+		break;
+	case 2:	// Green
+		VGA_DAC_SetEntry(0x7,0x00,0x26,0x00);
+		VGA_DAC_SetEntry(0xf,0x00,0x3f,0x00);
+		break;
+	}
+}
+
+static void write_hercules(Bitu port,Bitu val,Bitu /*iolen*/) {
 	switch (port) {
 	case 0x3b8: {
 		// the protected bits can always be cleared but only be set if the 
@@ -466,7 +516,7 @@ static void write_hercules(Bitu port,Bitu val,Bitu iolen) {
 		break;
 		}
 	case 0x3bf:
-		vga.herc.enable_bits=val;
+		vga.herc.enable_bits=(Bit8u)val;
 		break;
 	}
 }
@@ -476,7 +526,7 @@ static void write_hercules(Bitu port,Bitu val,Bitu iolen) {
 	return 0;
 } */
 
-Bitu read_herc_status(Bitu port,Bitu iolen) {
+Bitu read_herc_status(Bitu /*port*/,Bitu /*iolen*/) {
 	// 3BAh (R):  Status Register
 	// bit   0  Horizontal sync
 	//       1  Light pen status (only some cards)
@@ -507,6 +557,9 @@ Bitu read_herc_status(Bitu port,Bitu iolen) {
 void VGA_SetupOther(void) {
 	Bitu i;
 	memset( &vga.tandy, 0, sizeof( vga.tandy ));
+	vga.attr.disabled = 0;
+	vga.config.bytes_skip=0;
+
 	//Initialize values common for most machines, can be overwritten
 	vga.tandy.draw_base = vga.mem.linear;
 	vga.tandy.mem_base = vga.mem.linear;
@@ -523,10 +576,13 @@ void VGA_SetupOther(void) {
 		extern Bit8u int10_font_14[256 * 14];
 		for (i=0;i<256;i++)	memcpy(&vga.draw.font[i*32],&int10_font_14[i*14],14);
 		vga.draw.font_tables[0]=vga.draw.font_tables[1]=vga.draw.font;
+		MAPPER_AddHandler(CycleHercPal,MK_f11,0,"hercpal","Herc Pal");
 	}
 	if (machine==MCH_CGA) {
 		IO_RegisterWriteHandler(0x3d8,write_cga,IO_MB);
 		IO_RegisterWriteHandler(0x3d9,write_cga,IO_MB);
+		IO_RegisterWriteHandler(0x3db,write_tandy,IO_MB);
+		IO_RegisterWriteHandler(0x3dc,write_tandy,IO_MB);
 		MAPPER_AddHandler(IncreaseHue,MK_f11,MMOD2,"inchue","Inc Hue");
 		MAPPER_AddHandler(DecreaseHue,MK_f11,0,"dechue","Dec Hue");
 	}
@@ -580,4 +636,3 @@ void VGA_SetupOther(void) {
 	}
 
 }
-
