@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2009  The DOSBox Team
+ *  Copyright (C) 2002-2010  The DOSBox Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-/* $Id: vga_seq.cpp,v 1.24 2009/05/27 09:15:41 qbix79 Exp $ */
+/* $Id: vga_seq.cpp,v 1.24 2009-05-27 09:15:41 qbix79 Exp $ */
 
 #include "dosbox.h"
 #include "inout.h"
@@ -47,6 +47,8 @@ void write_p3c5(Bitu /*port*/,Bitu val,Bitu iolen) {
 			} else {
 				seq(clocking_mode)=val;
 			}
+			if (val & 0x20) vga.attr.disabled |= 0x2;
+			else vga.attr.disabled &= ~0x2;
 		}
 		/* TODO Figure this out :)
 			0	If set character clocks are 8 dots wide, else 9.
