@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2009  The DOSBox Team
+ *  Copyright (C) 2002-2010  The DOSBox Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-/* $Id: bios_disk.cpp,v 1.38 2009/05/27 09:15:42 qbix79 Exp $ */
+/* $Id: bios_disk.cpp,v 1.40 2009-08-23 17:24:54 c2woody Exp $ */
 
 #include "dosbox.h"
 #include "callback.h"
@@ -124,8 +124,9 @@ bool getSwapRequest(void) {
 void swapInNextDisk(bool pressed) {
 	if (!pressed)
 		return;
-	/* Hack/feature: rescan all disks as well */
 	DriveManager::CycleAllDisks();
+	/* Hack/feature: rescan all disks as well */
+	LOG_MSG("Diskcaching reset for normal mounted drives.");
 	for(Bitu i=0;i<DOS_DRIVES;i++) {
 		if (Drives[i]) Drives[i]->EmptyCache();
 	}
@@ -196,7 +197,7 @@ imageDisk::imageDisk(FILE *imgFile, Bit8u *imgName, Bit32u imgSizeK, bool isHard
 	active = false;
 	hardDrive = isHardDisk;
 	if(!isHardDisk) {
-		Bitu i=0;
+		Bit8u i=0;
 		bool founddisk = false;
 		while (DiskGeometryList[i].ksize!=0x0) {
 			if ((DiskGeometryList[i].ksize==imgSizeK) ||
@@ -309,6 +310,9 @@ static Bitu INT13_DiskHandler(void) {
 		if(imageDiskList[i]) any_images=true;
 	}
 
+	// unconditionally enable the interrupt flag
+	CALLBACK_SIF(true);
+
 	//drivenum = 0;
 	//LOG_MSG("INT13: Function %x called on drive %x (dos drive %d)", reg_ah,  reg_dl, drivenum);
 	switch(reg_ah) {
@@ -349,12 +353,15 @@ static Bitu INT13_DiskHandler(void) {
 			CALLBACK_SCF(true);
 			return CBRET_NONE;
 		}
-		if(!any_images && (reg_dh == 0)) { // Inherit the Earth cdrom (uses it as disk test)
-			reg_ah = 0;
-			CALLBACK_SCF(false);
-			return CBRET_NONE;
+		if (!any_images) {
+			// Inherit the Earth cdrom (uses it as disk test)
+			if (((reg_dl&0x80)==0x80) && (reg_dh==0) && ((reg_cl&0x3f)==1)) {
+				reg_ah = 0;
+				CALLBACK_SCF(false);
+				return CBRET_NONE;
+			}
 		}
-		if(driveInactive(drivenum)) {
+		if (driveInactive(drivenum)) {
 			reg_ah = 0xff;
 			CALLBACK_SCF(true);
 			return CBRET_NONE;
