@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2009  The DOSBox Team
+ *  Copyright (C) 2002-2010  The DOSBox Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-/* $Id: ems.cpp,v 1.62 2009/05/14 17:51:47 qbix79 Exp $ */
+/* $Id: ems.cpp,v 1.65 2009-10-11 17:11:52 c2woody Exp $ */
 
 #include <string.h>
 #include <stdlib.h>
@@ -40,11 +40,14 @@
 #define EMM_MAX_PAGES	(32 * 1024 / 16 )
 #define EMM_MAX_PHYS	4				/* 4 16kb pages in pageframe */
 
-#define EMM_VERSION		0x40
-#define GEMMIS_VERSION	0x0001	// Version 1.0
+#define EMM_VERSION			0x40
+#define EMM_MINOR_VERSION	0x00
+//#define EMM_MINOR_VERSION	0x30	// emm386 4.48
+#define GEMMIS_VERSION		0x0001	// Version 1.0
 
-#define NULL_HANDLE	0xffff
-#define	NULL_PAGE	0xffff
+#define EMM_SYSTEM_HANDLE	0x0000
+#define NULL_HANDLE			0xffff
+#define	NULL_PAGE			0xffff
 
 #define ENABLE_VCPI 1
 #define ENABLE_V86_STARTUP 0
@@ -71,6 +74,25 @@
 #define EMM_MOVE_OVLAPI			0x97
 #define EMM_NOT_FOUND			0xa0
 
+
+struct EMM_Mapping {
+	Bit16u handle;
+	Bit16u page;
+};
+
+struct EMM_Handle {
+	Bit16u pages;
+	MemHandle mem;
+	char name[8];
+	bool saved_page_map;
+	EMM_Mapping page_map[EMM_MAX_PHYS];
+};
+
+static EMM_Handle emm_handles[EMM_MAX_HANDLES];
+static EMM_Mapping emm_mappings[EMM_MAX_PHYS];
+static EMM_Mapping emm_segmentmappings[0x40];
+
+
 static Bit16u GEMMIS_seg; 
 
 class device_EMM : public DOS_Device {
@@ -144,8 +166,13 @@ bool device_EMM::ReadFromControlChannel(PhysPt bufptr,Bit16u size,Bit16u * retco
 			mem_writew(GEMMIS_addr+0x18d,0x0000);		// system handle
 			mem_writed(GEMMIS_addr+0x18f,0);			// handle name
 			mem_writed(GEMMIS_addr+0x193,0);			// handle name
-			mem_writew(GEMMIS_addr+0x197,0x0010);		// system handle
-			mem_writed(GEMMIS_addr+0x199,0x00110000);	// physical address
+			if (emm_handles[EMM_SYSTEM_HANDLE].pages != NULL_HANDLE) {
+				mem_writew(GEMMIS_addr+0x197,(emm_handles[EMM_SYSTEM_HANDLE].pages+3)/4);
+				mem_writed(GEMMIS_addr+0x199,emm_handles[EMM_SYSTEM_HANDLE].mem<<12);	// physical address
+			} else {
+				mem_writew(GEMMIS_addr+0x197,0x0001);		// system handle
+				mem_writed(GEMMIS_addr+0x199,0x00110000);	// physical address
+			}
 
 			/* fill buffer with import structure */
 			mem_writed(bufptr+0x00,GEMMIS_seg<<4);
@@ -155,31 +182,21 @@ bool device_EMM::ReadFromControlChannel(PhysPt bufptr,Bit16u size,Bit16u * retco
 			}
 		case 0x02:
 			if (size!=2) return false;
-			mem_writeb(bufptr+0x00,EMM_VERSION>>4);		// version 4.0
-			mem_writew(bufptr+0x01,EMM_VERSION&0x0f);
+			mem_writeb(bufptr+0x00,EMM_VERSION>>4);		// version 4
+			mem_writeb(bufptr+0x01,EMM_MINOR_VERSION);
+			*retcode=2;
+			return true;
+		case 0x03:
+			if (EMM_MINOR_VERSION < 0x2d) return false;
+			if (size!=4) return false;
+			mem_writew(bufptr+0x00,(Bit16u)(MEM_TotalPages()*4));	// max size (kb)
+			mem_writew(bufptr+0x02,0x80);							// min size (kb)
 			*retcode=2;
 			return true;
 	}
 	return false;
 }
 
-struct EMM_Mapping {
-	Bit16u handle;
-	Bit16u page;
-};
-
-struct EMM_Handle {
-	Bit16u pages;
-	MemHandle mem;
-	char name[8];
-	bool saved_page_map;
-	EMM_Mapping page_map[EMM_MAX_PHYS];
-};
-
-static EMM_Handle emm_handles[EMM_MAX_HANDLES];
-static EMM_Mapping emm_mappings[EMM_MAX_PHYS];
-static EMM_Mapping emm_segmentmappings[0x40];
-
 static struct {
 	bool enabled;
 	Bit16u ems_handle;
@@ -239,7 +256,7 @@ static Bit8u EMM_AllocateMemory(Bit16u pages,Bit16u & dhandle,bool can_allocate_
 static Bit8u EMM_AllocateSystemHandle(Bit16u pages) {
 	/* Check for enough free pages */
 	if ((MEM_FreeTotal()/ 4) < pages) { return EMM_OUT_OF_LOG;}
-	Bit16u handle = 0;	// emm system handle (reserved for OS usage)
+	Bit16u handle = EMM_SYSTEM_HANDLE;	// emm system handle (reserved for OS usage)
 	/* Release memory if already allocated */
 	if (emm_handles[handle].pages != NULL_HANDLE) {
 		MEM_ReleasePages(emm_handles[handle].mem);
@@ -664,7 +681,7 @@ static Bitu INT67_Handler(void) {
 		reg_ah=EMM_NO_ERROR;
 		break;
 	case 0x42:		/* Get number of pages */
-		reg_dx=MEM_TotalPages()/4;		//Not entirely correct but okay
+		reg_dx=(Bit16u)(MEM_TotalPages()/4);		//Not entirely correct but okay
 		reg_bx=EMM_GetFreePages();
 		reg_ah=EMM_NO_ERROR;
 		break;
@@ -1093,11 +1110,11 @@ static Bitu V86_Monitor() {
 				}
 				break;
 			case 0xe4:		// IN AL,Ib
-				reg_al=IO_ReadB(mem_readb((v86_cs<<4)+v86_ip+1));
+				reg_al=(Bit8u)(IO_ReadB(mem_readb((v86_cs<<4)+v86_ip+1))&0xff);
 				mem_writew(SegPhys(ss)+((reg_esp+0) & cpu.stack.mask),v86_ip+2);
 				break;
 			case 0xe5:		// IN AX,Ib
-				reg_ax=IO_ReadW(mem_readb((v86_cs<<4)+v86_ip+1));
+				reg_ax=(Bit16u)(IO_ReadW(mem_readb((v86_cs<<4)+v86_ip+1))&0xffff);
 				mem_writew(SegPhys(ss)+((reg_esp+0) & cpu.stack.mask),v86_ip+2);
 				break;
 			case 0xe6:		// OUT Ib,AL
@@ -1109,11 +1126,11 @@ static Bitu V86_Monitor() {
 				mem_writew(SegPhys(ss)+((reg_esp+0) & cpu.stack.mask),v86_ip+2);
 				break;
 			case 0xec:		// IN AL,DX
-				reg_al=IO_ReadB(reg_dx);
+				reg_al=(Bit8u)(IO_ReadB(reg_dx)&0xff);
 				mem_writew(SegPhys(ss)+((reg_esp+0) & cpu.stack.mask),v86_ip+1);
 				break;
 			case 0xed:		// IN AX,DX
-				reg_ax=IO_ReadW(reg_dx);
+				reg_ax=(Bit16u)(IO_ReadW(reg_dx)&0xffff);
 				mem_writew(SegPhys(ss)+((reg_esp+0) & cpu.stack.mask),v86_ip+1);
 				break;
 			case 0xee:		// OUT DX,AL
@@ -1167,13 +1184,15 @@ static Bitu V86_Monitor() {
 }
 
 static void SetupVCPI() {
+	vcpi.enabled=false;
+
+	vcpi.ems_handle=0;	// use EMM system handle for VCPI data
+
 	vcpi.enabled=true;
 
 	vcpi.pic1_remapping=0x08;	// master PIC base
 	vcpi.pic2_remapping=0x70;	// slave PIC base
 
-	/* Allocate one EMS-page for private VCPI-data in memory beyond 1MB */
-	EMM_AllocateMemory(1,vcpi.ems_handle,false);
 	vcpi.private_area=emm_handles[vcpi.ems_handle].mem<<12;
 
 	/* GDT */
@@ -1248,9 +1267,10 @@ class EMS: public Module_base {
 private:
 	/* location in protected unfreeable memory where the ems name and callback are
 	 * stored  32 bytes.*/
-	static Bit16u emsnameseg;
+	static Bit16u ems_baseseg;
 	RealPt old4b_pointer,old67_pointer;
-	CALLBACK_HandlerObject call_vdma,int67,call_vcpi,call_v86mon;
+	CALLBACK_HandlerObject call_vdma,call_vcpi,call_v86mon;
+	Bitu call_int67;
 
 public:
 	EMS(Section* configuration):Module_base(configuration){
@@ -1269,24 +1289,21 @@ public:
 			return;
 		}
 		BIOS_ZeroExtendedSize(true);
-		int67.Install(&INT67_Handler,CB_IRET,"Int 67 ems");
-		Bit16u call_int67=int67.Get_callback();
+
+		if (!ems_baseseg) ems_baseseg=DOS_GetMemory(2);	//We have 32 bytes
+
+		/* Add a little hack so it appears that there is an actual ems device installed */
+		char const* emsname="EMMXXXX0";
+		MEM_BlockWrite(PhysMake(ems_baseseg,0xa),emsname,(Bitu)(strlen(emsname)+1));
+
+		call_int67=CALLBACK_Allocate();
+		CALLBACK_Setup(call_int67,&INT67_Handler,CB_IRET,PhysMake(ems_baseseg,4),"Int 67 ems");
+		RealSetVec(0x67,RealMake(ems_baseseg,4),old67_pointer);
 
 		/* Register the ems device */
 		//TODO MAYBE put it in the class.
 		DOS_Device * newdev = new device_EMM();
 		DOS_AddDevice(newdev);
-	
-		/* Add a little hack so it appears that there is an actual ems device installed */
-		char const* emsname="EMMXXXX0";
-		if(!emsnameseg) emsnameseg=DOS_GetMemory(2);	//We have 32 bytes
-		MEM_BlockWrite(PhysMake(emsnameseg,0xa),emsname,(Bitu)(strlen(emsname)+1));
-
-		/* Copy the callback piece into the beginning, and set the interrupt vector to it*/
-		char buf[16];
-		MEM_BlockRead(CALLBACK_PhysPointer(call_int67),buf,0xa);
-		MEM_BlockWrite(PhysMake(emsnameseg,0),buf,0xa);
-		RealSetVec(0x67,RealMake(emsnameseg,0),old67_pointer);
 
 		/* Clear handle and page tables */
 		Bitu i;
@@ -1304,7 +1321,8 @@ public:
 			emm_segmentmappings[i].handle=NULL_HANDLE;
 		}
 
-		EMM_AllocateSystemHandle(4);	// allocate OS-dedicated handle (ems handle zero)
+		EMM_AllocateSystemHandle(8);	// allocate OS-dedicated handle (ems handle zero, 128kb)
+
 
 		if (!ENABLE_VCPI) return;
 
@@ -1315,6 +1333,8 @@ public:
 		/* Initialize private data area and set up descriptor tables */
 		SetupVCPI();
 
+		if (!vcpi.enabled) return;
+
 		/* Install v86-callback that handles interrupts occuring
 		   in v86 mode, including protection fault exceptions */
 		call_v86mon.Install(&V86_Monitor,CB_IRET,"V86 Monitor");
@@ -1351,6 +1371,7 @@ public:
 	~EMS() {
 		Section_prop * section=static_cast<Section_prop *>(m_configuration);
 		if (!section->Get_bool("ems")) return;
+
 		/* Undo Biosclearing */
 		BIOS_ZeroExtendedSize(false);
  
@@ -1361,15 +1382,18 @@ public:
 
 		/* Remove the emsname and callback hack */
 		char buf[32]= { 0 };
-		MEM_BlockWrite(PhysMake(emsnameseg,0),buf,32);
+		MEM_BlockWrite(PhysMake(ems_baseseg,0),buf,32);
 		RealSetVec(0x67,old67_pointer);
+
+		/* Release memory allocated to system handle */
+		if (emm_handles[EMM_SYSTEM_HANDLE].pages != NULL_HANDLE) {
+			MEM_ReleasePages(emm_handles[EMM_SYSTEM_HANDLE].mem);
+		}
+
 		/* Clear handle and page tables */
 		//TODO
 
-		if (!ENABLE_VCPI) return;
-
-		/* Free private data area in expanded memory */
-		EMM_ReleaseMemory(vcpi.ems_handle);
+		if ((!ENABLE_VCPI) || (!vcpi.enabled)) return;
 
 		if (cpu.pmode && GETFLAG(VM)) {
 			/* Switch back to real mode if in v86-mode */
@@ -1394,4 +1418,4 @@ void EMS_Init(Section* sec) {
 }
 
 //Initialize static members
-Bit16u EMS::emsnameseg = 0;
+Bit16u EMS::ems_baseseg = 0;
