@@ -1,10 +1,7 @@
-#define INLINE __forceinline
-
-#define VERSION "0.63"
-
+#define VERSION "0.74"
 
 /* Define to 1 to enable internal debugger, requires libcurses */
-#define C_DEBUG 1
+#define C_DEBUG 0
 
 /* Define to 1 to enable screenshots, requires libpng */
 #define C_SSHOT 1
@@ -22,11 +19,15 @@
 #define C_HEAVY_DEBUG 0
 
 /* The type of cpu this host has */
-#define C_HOSTCPU X86
+#define C_TARGETCPU X86
+//#define C_TARGETCPU X86_64
 
 /* Define to 1 to use x86 dynamic cpu core */
 #define C_DYNAMIC_X86 1
 
+/* Define to 1 to use recompiling cpu core. Can not be used together with the dynamic-x86 core */
+#define C_DYNREC 0
+
 /* Enable memory function inlining in */
 #define C_CORE_INLINE 0
 
@@ -53,6 +54,14 @@
 
 #define GCC_ATTRIBUTE(x) /* attribute not supported */
 #define GCC_UNLIKELY(x) (x)
+#define GCC_LIKELY(x) (x)
+
+#define INLINE __forceinline
+#define DB_FASTCALL __fastcall
+
+#if defined(_MSC_VER) && (_MSC_VER >= 1400) 
+#pragma warning(disable : 4996) 
+#endif
 
 typedef         double		Real64;
 /* The internal types */
